/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign;

import javax.swing.*;
import be.cardon.utils.Debug;
import java.util.List;
import java.util.ArrayList;
import java.net.URL;
import java.net.URI;
import java.io.*;
import be.cardon.utils.Path;
import java.util.Iterator;
import java.security.KeyStore;
import java.security.PrivateKey;
import be.cardon.sign.shared.KeyStoreEntry;
import be.cardon.sign.xmldsig.XMLSignerEngine;
import java.net.*;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import be.cardon.sign.applet.*;
import be.cardon.sign.wizard.WizardFrame;
import be.cardon.sign.communicator.DownloaderFile;



/**
 *
 * @author Rodolphe
 */
public class WebSignApplication {
    
    public final static String version = "06 november 2006 (b)";
    private final static String licenseFile = "license.txt";
    
    private static WebSignApplication _wsa;
    
    public List<DownloaderFile> downloaderFileList;
    public File targetDirectory;
    public File targetSignatureFile; //not used soon
    public KeyStoreEntry signingKeyStoreEntry; // loaded key store, alias, provider, chain
    public PrivateKey signingPrivateKey;
    public byte[] signature;
    private WebSignApplet applet;
    
    public boolean downloadComplete=false;
    private WizardFrame wizardFrame;
    public int maxDownloadSpeed = 1000000;
    
    /** Creates a new instance of WebSignAppletApplication */
    public WebSignApplication() {
        
        this._wsa = this;
        //init the main frame
        //to do ...use the thread safe method
        WizardFrame wizardFrame;
        wizardFrame = new WizardFrame();
        wizardFrame.setLocationRelativeTo(null);
        wizardFrame.setVisible(true);
        wizardFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.wizardFrame = wizardFrame;
        
    }
    
    /**creates a new instance of application, or returns the existing instance.*/
    public static WebSignApplication getInstance(){
        if(_wsa==null){
            throw new NullPointerException("WebSignApplication doe'snt exist.");
        }
        return _wsa;
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Debug.setDebug(true);
        WebSignApplication appl = new WebSignApplication();
        String license = appl.readLicense();
        System.out.println(license);
        
    }

    public void setApplet(WebSignApplet applet) {
        this.applet = applet;
    }  

    /**Construct the downloaderFileList.*/
    public void makeDownloaderFileList() throws InvalidAppletArgumentsException{
        SignatureSession session = this.applet.getSignatureSession();
        List<DownloaderFile> dfList = new ArrayList<DownloaderFile>();
        List<FileAppletParameter> faps = getFilesFromApplet();
        String scriptString = getScriptFromApplet();
        
        if(faps ==null || faps.size()== 0 || scriptString == null){
            throw new InvalidAppletArgumentsException("Empty file list or empty script URL");
        }
        
        if(scriptString.length()==0){
            throw new InvalidAppletArgumentsException("Empty script URL");
        }
        
        try{
            
            URL scriptURL = new URL(scriptString);
            String protocol=scriptURL.getProtocol();
            if(protocol==null){
                throw new InvalidAppletArgumentsException("Script protocol unreadable");
            }
            
            if(! (protocol.equalsIgnoreCase("http") || protocol.equalsIgnoreCase("https"))){
                throw new InvalidAppletArgumentsException("Script Invalid protocol : only http or https supported.");
            }
            if(protocol.equalsIgnoreCase("https")){
                throw new InvalidAppletArgumentsException("Script Invalid protocol : https currently not implemented.");
            }
            
            for(Iterator<FileAppletParameter> i = faps.iterator();i.hasNext();){
                FileAppletParameter fap = i.next();
                String relativePathString = fap.getRelativePath();
                Long  size = fap.getSize();
                
                Path path = Path.parse(relativePathString);
                
                String[] dirs = path.normalize().getDirectoriesArray();
                if(dirs.length>0){
                    if(dirs[0].equals("..")){
                        throw new InvalidAppletArgumentsException("Prohibited relative path with /../ :"+path.toString());
                    }
                }
                if(path.isAbsolutePath()){
                    throw new InvalidAppletArgumentsException("Prohibited absolute path :"+path.toString());
                }
                if(path.isDirectory()){
                    throw new InvalidAppletArgumentsException("Wrong path : may not be a directory :"+path.toString());
                }
                
                File relativeFile = new File(path.toString());
                
                String relativeFileEncoded = new URI(path.toString()).toString();
                URL sourceURL = new URL(scriptURL.toString() 
                    + "?file=" +relativeFileEncoded
                    + "&session=" + session.getSessionID()
                    + "&key=" + session.getSessionKey());
                DownloaderFile df = new DownloaderFile(sourceURL, relativeFile, size);
                
                //add request for sha-1 digest message
                df.addDigest("SHA-1", null);
                
                dfList.add(df);
            }
            
            this.downloaderFileList = dfList;
            
        }catch(Exception e){
            throw new InvalidAppletArgumentsException(e);
        }
    }
    
    /**returns the files from the applet.*/
    private List<FileAppletParameter> getFilesFromApplet() 
    throws InvalidAppletArgumentsException{
        return this.applet.getFiles();
    }
    
    /**returns the script from the applet.*/
    private String getScriptFromApplet()
    throws InvalidAppletArgumentsException{
        return this.applet.getScript();
    }
    
    
    public void sign() throws Exception{
        
        if(downloaderFileList == null){
            throw new NullPointerException("downloaderFileList");
        }

        this.getPrivateKey();
        //install provider in first position
        java.security.Security.insertProviderAt(this.signingKeyStoreEntry.getProvider(), 0);
        
        File[] files2sign = new File[downloaderFileList.size()];
        int counter=0;
        for(Iterator<DownloaderFile> it = downloaderFileList.iterator();it.hasNext();counter++){
            files2sign[counter]= it.next().getLocalAbsoluteFile();
        }
        
        URI baseURI = targetDirectory.getAbsoluteFile().toURI();
        Certificate[] chain2 = this.signingKeyStoreEntry.getCertificateChain();
        List<X509Certificate> chain = new ArrayList<X509Certificate>();
        for(int i=0;i<chain2.length;i++){
            chain.add((X509Certificate)chain2[i]);
        }
        Calendar now = Calendar.getInstance();
        ByteArrayOutputStream os = new ByteArrayOutputStream(1000);
        Writer writer = XMLSignerEngine.StandardOutputWriter;
        
        XMLSignerEngine.Sign2XML(files2sign,baseURI,this.signingPrivateKey, chain,
                now,os,writer);
        this.signature = os.toByteArray();
    }
    
    /**return status.*/
    public int uploadSignature() throws Exception{
        SignatureSession session = this.applet.getSignatureSession();
        
        String script = this.getScriptFromApplet();
        URI scriptURI = new URI(script);
        String query = scriptURI.getQuery();
        String params = "signupload=1" +
                "&session="+session.getSessionID()+
                "&key="+session.getSessionKey();
        if(query==null){
            query = params;
        }else{
            query = query + "&" + params;
        }
       
        URI uploadURI = new URI(scriptURI.getScheme(),
                scriptURI.getAuthority(),
                scriptURI.getPath(), query,
                scriptURI.getFragment());
        Debug.println("uploadUR = "+uploadURI);
        return be.cardon.sign.communicator.SignatureUpload.uploadSignature(uploadURI, this.signature);
    }
    
    private PrivateKey getPrivateKey(char[] pass) throws Exception{
        if(signingKeyStoreEntry==null){
            throw new NullPointerException("signingKeyStoreEntry");
        }
        String alias = signingKeyStoreEntry.getAlias();
        return (PrivateKey) signingKeyStoreEntry.getLoadedKeyStore().getKey(alias, pass);
    }
    
    /**get private key, GUI ask password if needed.*/
    public void getPrivateKey() throws Exception{
        PrivateKey key=null;
        try{
            key=getPrivateKey(null);
        }catch(Exception e){
            boolean firstAsk = true;
            while(key==null){
                char[] pass = be.cardon.sign.shared.DialogBox.askPassword(
                        null, "Enter the private key password.",firstAsk);
                firstAsk =false;
                if(pass==null){throw new Exception("Operation cancelled");} //user cancelled
                firstAsk = false;
                try{
                    key=getPrivateKey(pass);
                }catch(Exception e2){
                    //loop
                }
            }
        }
        if(key==null){
            throw new NullPointerException("key is null !");
        }
        this.signingPrivateKey = key;
        
    }
    
    public void quit(){
        wizardFrame.dispose();
    }
    
        //before: refresh, then quit => bug : sometimes, Internet Explorer freezes.
        //now : quit, then refresh => no bug anymore ??
    public void refreshPage(){
        try{
        this.quit();
        this.applet.URLRefresh();
        }catch(Exception e){
            be.cardon.sign.shared.DialogBox.showExceptionBox(null, e);
        }

    }

    public String readLicense(){
        try{
            InputStream in = this.getClass().getClassLoader().getResourceAsStream(this.licenseFile);
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String content= new String();
            while(true){
                   String line = reader.readLine();
                   if(line==null){
                       reader.close();
                       return content;
                   }
                   content = content + line + "\n";
             }
            
        }catch(Exception e){
            e.printStackTrace();
            return new String("License file not found.");
        }
    }
    
}
