/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 * 
 * TO DO : applet parameters should be checked at the beginning of the application.
 */



package be.cardon.sign.applet;

import javax.swing.*;
import be.cardon.utils.Debug;
import be.cardon.sign.wizard.WizardFrame;
import be.cardon.sign.*;
import java.util.ArrayList;
import java.util.List;
import java.net.URL;

/**
 *
 * @author Rodolphe
 */
public class WebSignApplet extends javax.swing.JApplet implements Runnable{
    
    private JPanel appletPanel;
    private JTextArea appletText;
    private WizardFrame wizardFrame;
    private WebSignApplication application;
    
    //interface definition
    //parameters are case insensitive (see Applet.getParameter)
    private static final String SCRIPT_PARA = "script";
    private static final String SESSION_ID_PARA = "id";
    private static final String SESSION_KEY_PARA = "key";
    private static final String FILES_NUM_PARA = "numberOfFiles";
    private static final String FILE_PATH_PREFIX = "filePath";
    private static final String FILE_SIZE_PREFIX = "fileSize";
    private static final String FILE_HASH_TYPE_PREFIX = "fileHashType";
    private static final String FILE_HASH_VALUE_PREFIX = "fileHashValue";
    private static final String URL_REFRESH_PAGE = "urlRefreshPage";
    private static final String URL_REFRESH_TARGET = "urlRefreshTarget";
    
    /** Creates a new instance of WebSignApplet */
    public WebSignApplet() {
        
    }
    
    public void init(){
        Debug.setDebug(true);
        appletPanel = new JPanel();
        appletText = new JTextArea();
        appletText.setFont(new JLabel().getFont());
        appletText.setOpaque(false);
        appletText.setEditable(false);
        appletText.setWrapStyleWord(true);
        appletText.setLineWrap(true);
        
        appletText.setText("A new window will appear...");
        appletPanel.add(appletText);
        this.getContentPane().add(appletPanel);

    }
    

    
    public void start(){
        if(application==null){
            Thread thread = new Thread(this);
            thread.start();
        }
    }
    
    public void run(){
        application = new WebSignApplication();
        application.setApplet(this);
    }
    
    public void URLRefresh()throws InvalidAppletArgumentsException{
        String target = this.getMustExistParameter(URL_REFRESH_TARGET);
        String URLRefreshString = this.getMustExistParameter(URL_REFRESH_PAGE);
        URL url;
        try{
        url = new URL(URLRefreshString);
        }catch(Exception e){
            throw new InvalidAppletArgumentsException(e);
        }
        this.getAppletContext().showDocument(url, target);
    }
    
    public void stop(){
        //do nothing
    }
    public void destroy(){
        //TO DO : close the application... (ask confirmation)
    }
    
    /**return the remote HTTP service.*/
    public String getScript() throws InvalidAppletArgumentsException{
        String script = this.getMustExistParameter(SCRIPT_PARA);
        return script;
    }
    
    public List<FileAppletParameter> getFiles() throws InvalidAppletArgumentsException{
        try{
            String numberOfFilesString = this.getMustExistParameter(FILES_NUM_PARA);
            int numberOfFiles = Integer.parseInt(numberOfFilesString);
            if(numberOfFiles<1){
                throw new InvalidAppletArgumentsException(FILES_NUM_PARA
                        + " parameter must be > 0");
            }
            
            List<FileAppletParameter> faps = new ArrayList<FileAppletParameter>();
            for(int i=1;i<=numberOfFiles;i++){
                String suffix = Integer.toString(i);
                String filePath = this.getMustExistParameter(FILE_PATH_PREFIX + suffix);
                FileAppletParameter fap = new FileAppletParameter(filePath);
                String fileSizeString = this.getParameter(FILE_SIZE_PREFIX + suffix);
                if(fileSizeString!=null){
                    long fileSize = Long.parseLong(fileSizeString);
                    if(fileSize<0){
                        throw new InvalidAppletArgumentsException(FILE_SIZE_PREFIX + suffix
                                + "may not be negative.");
                    }
                    fap.setSize(fileSize);
                }
                String fileHashType = this.getParameter(FILE_HASH_TYPE_PREFIX + suffix);
                if(fileHashType!=null){
                    String fileHashValue = this.getMustExistParameter(FILE_HASH_VALUE_PREFIX + suffix);
                    fap.setHash(fileHashType, fileHashValue);
                }
                faps.add(fap);
            }
            return faps;
        }catch(NumberFormatException e){
            throw new InvalidAppletArgumentsException(e);
        }
    }
    
    public SignatureSession getSignatureSession() throws InvalidAppletArgumentsException{
        String sessionID = this.getMustExistParameter(SESSION_ID_PARA);
        String sessionKey =  this.getMustExistParameter(SESSION_KEY_PARA);
        return new SignatureSession(sessionID, sessionKey);
    }
    
    /**returns the parameter. Throws an InvalidAppletArguments
     *if the parameter doesn't exist.*/
    private String getMustExistParameter(String parameterName)
    throws InvalidAppletArgumentsException{
        String parameterValue = this.getParameter(parameterName);
        if(parameterValue==null){
            throw new InvalidAppletArgumentsException(parameterName
                    + " parameter not found");
        }
        return parameterValue;
    }
    
    
    
    
}
