/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.communicator;

import java.util.List;
import java.util.ArrayList;
import java.io.*;
import java.net.*;
import java.util.Iterator;
import be.cardon.utils.Debug;
import be.cardon.sign.applet.*;
import be.cardon.sign.*;
/**
 *
 * @author Rodolphe
 */
public class Downloader implements Runnable{
    
    public final int STATUS_NOT_STARTED = 0;
    public final int STATUS_DOWNLOADING = 2;
    public final int STATUS_FINISH = 3;
    public final int STATUS_ERROR = 4;
    
    private int status;
    //private String errorMessage;
    private DownloaderFile currentDownloaderFile;
    private List<DownloaderFile> downloaderFileList;
    private long totalDownloaded=0; // updated between each file;
    
    private Exception lastException;
    private Thread downloadThread;
    
    /** Creates a new instance of Downloader */
    public Downloader(List<DownloaderFile> downloaderFileList) {
        if(downloaderFileList==null){
            throw new NullPointerException("downloaderFileList");
        }
        this.downloaderFileList = downloaderFileList;
        this.setStatus(STATUS_NOT_STARTED);
        totalDownloaded = 0;
    }
       
    public void interrupt(){
        if(this.getStatus() == STATUS_DOWNLOADING && currentDownloaderFile!=null){
            currentDownloaderFile.interrupt();
        }
    }
    
    public synchronized int getStatus(){
        return this.status;
    }
    private  synchronized void setStatus(int status){
        this.status = status;
    }
    
    /*public String getErrorMessage(){
        return this.errorMessage;
    }*/
    public void downloadAll(){
        downloadThread = new Thread(this);;
        downloadThread.start();
    }
    //synchron return
    public void run(){
        try{
            this.setStatus(STATUS_DOWNLOADING);
            for(Iterator<DownloaderFile> it = downloaderFileList.iterator();it.hasNext();){
                currentDownloaderFile= it.next();
                currentDownloaderFile.download();
                totalDownloaded += currentDownloaderFile.getDownloadedSize();
            }
            this.setStatus(STATUS_FINISH);
        } catch(Exception e){
            this.setStatus(this.STATUS_ERROR);
            this.lastException = e;
        }
    }
    
    public DownloaderFile getCurrentDownloaderFile(){
        return this.currentDownloaderFile;
    }
    
    public int getTotalLength(){
        int count=0;
        for(Iterator<DownloaderFile> it = downloaderFileList.iterator();it.hasNext();){
            DownloaderFile  df = it.next();
            long size = df.getSize();
            if(size==-1){
                return -1;
            }
            count+=size;
        }
        return count;
    }
    
    public long getCurrentFileLength(){
        if(this.currentDownloaderFile==null){
            return -1;
        }else{
            return currentDownloaderFile.getSize();
        }
    }
    
    public long getCurrentFileDownloaded(){
        if(this.currentDownloaderFile==null){
            return 0;
        }else{
            return currentDownloaderFile.getDownloadedSize();
        }
    }
    
    public long getTotalDownloadedFile(){
        if(this.status==this.STATUS_FINISH || this.currentDownloaderFile==null) {
            return this.totalDownloaded;
        }else{
            return this.totalDownloaded + currentDownloaderFile.getDownloadedSize();
        }
    }
    public Exception getLastException(){
        return this.lastException;
    }
}
