/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.shared;


import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import be.cardon.utils.Debug;


/**
 *
 * @author Rodolphe
 */
public class DialogBox {
    
    /** Creates a new instance of WizardDialogBox */
    public DialogBox() {
    }
    public static void showWarningBox(JFrame parentFrame, String message){
        //to do : change the icon...etc...
        JOptionPane.showMessageDialog(parentFrame, message, "Warning", JOptionPane.WARNING_MESSAGE);
    }
    
    public static void showExceptionBox(JFrame parentFrame, String name,String  title, String message, Exception e){
        ExceptionDialog diag =
                new ExceptionDialog(parentFrame, name,title, message,e);
    }
    
    public static void showExceptionBox(JFrame parentFrame, String name,String  title, String message){
        ExceptionDialog diag =
                new ExceptionDialog(parentFrame, name,title, message,null);
    }

    public static void showExceptionBox(JFrame parentFrame, String title, Exception e){
        String message = e.getMessage();
        showExceptionBox(parentFrame, "Exception", title, message, e);}
    
    
    public static void showExceptionBox(JFrame parentFrame, Exception e){
        showExceptionBox(parentFrame, "An Exception occurs.", e);
    }
    
    public static char[] askPassword(JFrame parentFrame, String message, boolean firstAsk){
        Debug.println("message="+message);
        GUIPasswordDialog frame = new GUIPasswordDialog(parentFrame, message, firstAsk);
        frame.setVisible(true);
        Debug.println("DialogBox: password continous");
        char[] pass = null;
        if(frame.chooseOK){
            Debug.println("DialogBox: getPassword");
            pass = frame.getPassword();
        }
        frame.dispose();
        return pass;
    }
    
    public static void main(String[] args){
        try{
            Integer.parseInt("it is'nt a good interger !!");
        }catch(Exception e){
            JFrame parentFrame = null;
            showExceptionBox( parentFrame,  e);
            showExceptionBox( parentFrame,  "Error", "You got an error:",e.getMessage());
            showWarningBox(parentFrame, "You got an warning.");
            char[] pass=askPassword(parentFrame, "password 1", true);
            System.out.println(pass);
            char[] pass2=askPassword(parentFrame, "password 2", false);
            System.out.println(pass2);
            
        }
    }
}


class ExceptionDialog extends JDialog implements ActionListener{
    
    //private JButton detailsButton;
    private JButton okButton;
    private Icon icon;
    private JLabel titleLabel;
    private JTextArea messageArea;
    private JScrollPane detailsPanel;
    private JTextArea detailsMessage;
    private JDialog dialog;
    
    public ExceptionDialog(JFrame parentFrame, String name,String  title, String message, Exception e){
        super(parentFrame, name, true); //modal
        
        dialog = this;
        Icon icon = GUIUtils.getImageIcon("images/exception.gif","Exception");
        
        JLabel iconLabel = new JLabel();
        iconLabel.setIcon(icon);
        iconLabel.setAlignmentY(0.0f);
        
        titleLabel = new JLabel(title);
        Font defaultFont = new JLabel().getFont();
        titleLabel.setFont(defaultFont.deriveFont(Font.BOLD));
        titleLabel.setAlignmentX(0.0f);
        
        messageArea = new JTextArea(5, 30);
        messageArea.setEditable(false);
        messageArea.setFont(defaultFont.deriveFont(Font.PLAIN));
        messageArea.setOpaque(false);
        messageArea.setLineWrap(true);
        messageArea.setWrapStyleWord(true);
        messageArea.setText(message);
        messageArea.setAlignmentX(0.0f);
        
        JPanel titleAndMessagePanel = new JPanel();
        titleAndMessagePanel.setLayout(new BoxLayout(titleAndMessagePanel, BoxLayout.Y_AXIS));
        titleAndMessagePanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        titleAndMessagePanel.add(titleLabel);
        titleAndMessagePanel.add(messageArea);
        titleAndMessagePanel.setAlignmentY(0.0f);
        
        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
        
        topPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        topPanel.add(iconLabel);
        topPanel.add(titleAndMessagePanel);
        topPanel.setMaximumSize(new Dimension(Integer.MAX_VALUE, 75));
        topPanel.setPreferredSize(new Dimension(400, 75));
        //topPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        
        //detailsButton = new JButton("Details...");
        //detailsButton.addActionListener(this);
        
        Box detailBox=null;
        if(e!=null){
            detailsMessage = new JTextArea(getExceptionDetails(e));
            detailsMessage.setEditable(true);
            detailsMessage.setFont(defaultFont.deriveFont(Font.PLAIN));
            detailsMessage.setOpaque(false);
            //detailsMessage.setText();
            detailsMessage.setAlignmentX(0.0f);
            JScrollPane detailsPanel = new JScrollPane(detailsMessage);
            //detailsPanel.setPreferredSize((new Dimension(500,Integer.MAX_VALUE)));
            detailsPanel.setBorder(BorderFactory.createTitledBorder("Details"));
            
            detailBox = Box.createVerticalBox();
            detailBox.add(detailsPanel);
            detailBox.setPreferredSize(new Dimension(500,120));
            detailBox.setMaximumSize(new Dimension(Integer.MAX_VALUE,100));
        }
        
        okButton = new JButton("OK");
        okButton.addActionListener(this);
        JPanel mainPanel = new JPanel(new GridBagLayout());
        mainPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        GridBagConstraints c = new GridBagConstraints();
        
        c.gridx=0;
        c.gridy=0;
        c.anchor = c.WEST;
        c.fill = c.HORIZONTAL;
        mainPanel.add(topPanel, c);
        
        if(e!=null){
        c.gridy=1;
        c.anchor = c.CENTER;
        c.fill= c.NONE;
        mainPanel.add(detailBox, c);
        }
        
        c.gridy=2;
        c.anchor = c.CENTER;
        c.fill = c.NONE;
        
        mainPanel.add(okButton, c);
        
        
        this.getContentPane().add(mainPanel);
        
        
        //this.setPreferredSize(new Dimension(600,400));
        //this.setMaximumSize(new Dimension(600,400));
        this.pack();
        this.setLocationRelativeTo(parentFrame);
        this.setResizable(false);
        this.setVisible(true);
    }
    
    public void actionPerformed(ActionEvent event){
        Object source = event.getSource();
        /*if(source==detailsButton){
            displayDetails();
            return;
        }*/
        if(source==okButton){
            this.dispose();
        }
    }
    public void processWindowEvent(WindowEvent e) {
        if(e.getID()==e.WINDOW_CLOSING){
            this.dispose();
        }
        super.processWindowEvent(e);
    }
    
    public String getExceptionDetails(Throwable e){
        StackTraceElement[] stack = e.getStackTrace();
        String stackString= e.toString()+"\n";
        if(stack==null ||stack.length==0){return stackString;}
        stackString += "\nStack details:\n";
        for(int i=0;i<stack.length;i++){
            stackString += stack[i].toString()+"\n";
        }
        if(e.getCause()!=null){
            stackString += "\n\nCause:\n" + this.getExceptionDetails(e.getCause());
        }
        return stackString;
    }
    
}