/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.shared;

import be.cardon.utils.Path;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
/**
 *
 * @author Rodolphe
 */
public abstract class FileHierarchyEntry{
    
    
    private String name;
    private AbstractDirectory parent=null;
    private Object property=null;
    
    public FileHierarchyEntry(String name){
        this.name = name;
    }
    public String getName(){
        return this.name;
    }
    public String toString(){
        return this.getName();
    }
    public boolean isFile(){
        if(this instanceof AbstractFile){
            return true;}
        return false;
    }
    public boolean isDirectory(){
        if(this instanceof AbstractDirectory){
            return true;}
        return false;
    }
    public boolean isRoot(){
        if(this.parent==null){
            return true;
        }else{
            return false;
        }
    }
    public AbstractDirectory getParent(){
        return this.parent;
    }
    public void setParent(AbstractDirectory parent){
        this.parent = parent;
    }
    public void setProperty(Object obj){
        this.property  =obj;
    }
    public Object getProperty(){
        return this.property;
    }
    abstract Path getPath();
    
    abstract Path getPathWithoutRoot();
    
    public static AbstractDirectory newAbstractDirectory(AbstractDirectory parent, 
            String name, boolean caseSensitive)
        throws FileHierarchyException{
        return new AbstractDirectory(parent, name, caseSensitive);
    }
    
    
    
    public static class AbstractDirectory extends FileHierarchyEntry{
        private boolean caseSensitive;
        private List<FileHierarchyEntry> entries;
        
        public AbstractDirectory(AbstractDirectory parent, String name, boolean caseSensitive)
        throws FileHierarchyException{
            super(name);
            if(parent!=null){
                if(!parent.add(this)){
                    throw new FileHierarchyDuplicateException("Duplicate: "
                            +name+"in parent:"+parent.getPath());
                }
            }
            this.caseSensitive = caseSensitive;
            this.entries = new ArrayList<FileHierarchyEntry>();
        }
        
        /**return false if an entry already exists.*/
        public boolean add(List<FileHierarchyEntry> entries){
            boolean result = true;
            for(Iterator<FileHierarchyEntry> it=entries.iterator();it.hasNext();){
                if(!this.add(entries)){
                    result = false;
                }
            }
            return result;
        }
        
        /**add an entry if not already present (return true), otherwise do nothing (return false).*/
        public boolean add(FileHierarchyEntry entry){
            for(Iterator<FileHierarchyEntry> it=this.entries.iterator();it.hasNext();){
                if(this.caseSensitive
                        && it.next().getName().equals(entry.getName())){
                    return false;
                }else if(!this.caseSensitive
                        && it.next().getName().equalsIgnoreCase(entry.getName())){
                    return false;
                }
                
            }
            this.entries.add(entry);
            entry.setParent(this);
            return true;
        }
        public FileHierarchyEntry getChild(int index){
            return this.entries.get(index);
        }
        public int getChildCount(){
            return this.entries.size();
        }
        public Path getPath(){
            List<String> dirs = new java.util.ArrayList<String>();
            dirs.add(this.getName());
            AbstractDirectory parent = this.getParent();
            while(parent!=null){
                dirs.add(0, parent.getName());
                parent = parent.getParent();
            }
            return new Path(dirs, null, false);
        }
        public Path getPathWithoutRoot(){
            List<String> dirs = new java.util.ArrayList<String>();
            AbstractDirectory parent = this;
            while(!parent.isRoot()){
                dirs.add(0,parent.getName());
                parent = parent.getParent();
            }
            return new Path(dirs, null, false);
        }
    }
    
    public static class AbstractFile extends FileHierarchyEntry{

        private long size;
        
        public AbstractFile(AbstractDirectory parent, String name, long size)
        throws FileHierarchyException{
            super(name);
            this.size = size;
            if(parent!=null){
                if(!parent.add(this)){
                    throw new FileHierarchyDuplicateException("Duplicate file: "
                            +name+"in parent:"+parent.getPath());
                }
            }
        }
    
        public AbstractFile(AbstractDirectory parent, String name)
        throws FileHierarchyException{
            this(parent, name, -1);
        }
        
        public long getSize(){
            return this.size;
        }
        
        public Path getPath(){
            if(getParent()==null){
                return new Path(null, this.getName(), false);
            }else{
                return new Path(getParent().getPath().getDirectoriesList(),
                        this.getName(), false);
            }
        }
        
        public Path getPathWithoutRoot(){
            if(getParent()==null){
                return new Path(null, this.getName(), false);
            }else{
                return new Path(getParent().getPathWithoutRoot().getDirectoriesList(),
                        this.getName(), false);
            }
        }
    }
}
