/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.shared;

import be.cardon.utils.Debug;
import javax.swing.table.*;
import javax.swing.event.*;
import be.cardon.sign.shared.FileHierarchyEntry.AbstractDirectory;
import be.cardon.sign.shared.FileHierarchyEntry.AbstractFile;
/**
 *
 * @author Rodolphe
 */
public class FileHierarchyTableModel extends AbstractTableModel{
    
    private final String[] columnNames = {"Filename",   "Size",  "Path"};
    private AbstractDirectory abstractDirectory;
    
    /** Creates a new instance of FileHierarchyTableModel */
    public FileHierarchyTableModel(AbstractDirectory abstractDirectory) {
        this.abstractDirectory = abstractDirectory;
    }
    
    public void setAbstractDirectory(AbstractDirectory abstractDirectory){
        this.abstractDirectory = abstractDirectory;
        this.fireTableDataChanged();
    }
    
    public String getColumnName(int col) {
        return columnNames[col].toString();
    }
    
    public int getRowCount() {
        return abstractDirectory.getChildCount();
    }
    
    public int getColumnCount() { return columnNames.length; }
    
    public Object getValueAt(int row, int col) {
        FileHierarchyEntry entry = abstractDirectory.getChild(row);
        
        switch(col){
            case 0: return entry.getName();
            case 1:
            {
                String size="";
                if(entry.isFile()){
                    AbstractFile file = (AbstractFile)entry;
                    if(file.getSize()!=-1){
                        size = be.cardon.utils.FileUtils.getFriendlySize(file.getSize());
                    }else{
                        size = "??";
                    }
                }
                return size;
            }
            case 2: return entry.getPathWithoutRoot();
        }
        return null;
    }

    public FileHierarchyEntry getFileHierarchyEntry(int row){
        FileHierarchyEntry entry = abstractDirectory.getChild(row);
        return entry;
    }
}
