/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.shared;


import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.io.File;
import be.cardon.utils.Debug;
import be.cardon.utils.Path;
import be.cardon.sign.shared.FileHierarchyEntry.AbstractDirectory;
import be.cardon.sign.shared.FileHierarchyEntry.AbstractFile;
/**
 *
 * @author Rodolphe
 */
public class FileHierarchyTreeModel implements TreeModel{
    
    private boolean caseSensitive;
    private List<TreeModelListener> listeners; //not used (becase invariable model)
    private AbstractDirectory root;
    private String separator;

    public FileHierarchyTreeModel(AbstractDirectory root, boolean caseSensitive)
    throws FileHierarchyException{
        this.caseSensitive = caseSensitive;
        this.root = root;
        this.listeners = new ArrayList<TreeModelListener>();
        this.separator = be.cardon.utils.OperatingSystem.getSeparator();
    }

    
    public void addFileToRoot(File file, Object property, long size) throws FileHierarchyException{
        try{
            
            Path path = Path.parse(file.getPath(), this.separator);
            if(path.isAbsolutePath()){
                throw new FileHierarchyException("Path may not be absolute: "+path);
            }
            if(!path.isFile()){
                throw new FileHierarchyException("Must be a file name: "+path);
            }
            List<String> dirs=path.getDirectoriesList();
            AbstractDirectory abstractDir = root;
            AbstractDirectory abstractSubDir;
            boolean result;
            for(Iterator<String> it = dirs.iterator();it.hasNext();){
                String dirName = it.next();
                abstractSubDir = new AbstractDirectory(null, dirName, this.caseSensitive);
                result = abstractDir.add(abstractSubDir); //don't check duplicates.
                abstractDir = abstractSubDir;
            }
            //check for duplicate
            AbstractFile abstractFile= new AbstractFile(abstractDir, path.getFilename(), size);
            abstractFile.setProperty(property);
            
        }catch(be.cardon.utils.PathParserException e){
            throw new FileHierarchyException(e);
        }
    }
    
    /**
     * @inheritDoc
     */
    public void addTreeModelListener(TreeModelListener l) {
        //
    }
    
    /**
     * @inheritDoc
     */
    public Object getChild(Object parent, int index){
        try{
            if(parent instanceof AbstractDirectory){
                AbstractDirectory parentDir = (AbstractDirectory) parent;
                int countAll = parentDir.getChildCount();
                int countDirs = 0;
                for(int i = 0; i<countAll;i++){
                    if(parentDir.getChild(i).isDirectory()){
                        if(index==countDirs){
                            return parentDir.getChild(i);
                        }
                        countDirs++;
                    }
                }
                return null;
            }else{
                return null;
            }
        }catch(Exception e){
            e.printStackTrace();
            return null;
        }
    }
    
    /**
     * @inheritDoc
     * count for directories only !
     */
    public int getChildCount(Object parent){
        try{
            if(parent instanceof AbstractDirectory){
                AbstractDirectory parentDir = (AbstractDirectory) parent;
                int countAll = parentDir.getChildCount();
                int countDirs = 0;
                for(int i = 0; i<countAll;i++){
                    if(parentDir.getChild(i).isDirectory()){
                        countDirs++;
                    }
                }
                return countDirs;
            }else{
                return 0;
            }
        }catch(Exception e){
            e.printStackTrace();
            return 0;
        }
        
    }
    
    /**
     * @inheritDoc
     */
    public int getIndexOfChild(Object parent, Object child){
        try{
            if(parent==null || child == null){
                return -1;}
            if(!(parent instanceof AbstractDirectory)){
                return -1;}
            
            AbstractDirectory ADparent = (AbstractDirectory)parent;
            int count = ADparent.getChildCount();
            for(int i =0; i<count; i++){
                if(ADparent.getChild(i).equals(ADparent)){
                    return i;
                }
            }
            return -1;
        }catch(Exception e){
            e.printStackTrace();
            return -1;
        }
    }
    /**
     * @inheritDoc
     */
    public Object getRoot() {
        return this.root;
    }
    /**
     * @inheritDoc
     * Leaf : don't have children
     */
    public boolean isLeaf(Object node) {
        if(node instanceof AbstractDirectory){
            return false;
        }else{
            return true;
        }
    }
    /**
     * @inheritDoc
     */
    public void removeTreeModelListener(TreeModelListener l) {
    }
    /**
     * @inheritDoc
     */
    public void valueForPathChanged(TreePath path, Object newValue) {
        Debug.println("FileHierarchyModel valueForPathChanged not implemented");
    }    
}
