/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
*/

package be.cardon.sign.shared;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import be.cardon.utils.Debug;


public class GUIPasswordDialog extends JDialog implements ActionListener{
    
    private JPasswordField passField;
    private JButton okButton;
    private JButton cancelButton;
    public boolean chooseOK;
    private boolean firstAsk;
    private String message;
    private Frame parentFrame;
    
    
    public char[] getPassword(){
        return passField.getPassword();
    }
    
    public GUIPasswordDialog(JFrame parentFrame, String message, boolean firstAsk){
        super(parentFrame, "Password dialog", true);
        if(parentFrame==null){
            Debug.println("PasswordDialog: parentFrame=null");
        }
        this.parentFrame = parentFrame;
        this.firstAsk = firstAsk;
        this.message = message;
        
        Box box = Box.createVerticalBox();
        box.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        
        String text;
        if(this.firstAsk){
            text = message;
        }else{
            text = "Wrong Password or IO error (invalid file format).\n" +message;
        }
        JTextArea description = GUIUtils.newTextAreaNote(this, text);
        description.setAlignmentX(0.5f);
        description.setPreferredSize(new Dimension(300, 40));
        
        passField = new JPasswordField(10);
        passField.setEchoChar('*');
        passField.setActionCommand("OK");
        passField.addActionListener(this);
        passField.setAlignmentX(0.5f);
        passField.addKeyListener(this.keyListener);
        
        okButton = new JButton("OK");
        okButton.setActionCommand("OK");
        okButton.addActionListener(this);
        okButton.addKeyListener(this.keyListener);
        okButton.setAlignmentX(0.5f);
        
        cancelButton = new JButton("Cancel");
        cancelButton.setActionCommand("CANCEL");
        cancelButton.addActionListener(this);
        cancelButton.addKeyListener(this.keyListener);
        cancelButton.setAlignmentX(0.5f);
        
        box.add(description);
        box.add(passField);
        box.add(okButton);
        
        //this.setDefaultCloseOperation(this.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                ;
            }
        });
        
        this.addKeyListener(keyListener);
        
        this.getContentPane().add(box);
        this.pack();
        this.setLocationRelativeTo(this.parentFrame);
    }
    
    KeyListener keyListener = (new KeyAdapter() {
        public void keyPressed(KeyEvent e){
            if (e.getKeyCode() == KeyEvent.VK_ESCAPE ||
                    (e.getSource() == cancelButton
                    && e.getKeyCode() == KeyEvent.VK_ENTER)){
                chooseOK = false;
                setVisible(false);
            }
            if (e.getSource() == okButton &&
                    e.getKeyCode() == KeyEvent.VK_ENTER){
                chooseOK = true;
                setVisible(false);
            }
        }
    });
    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();
        
        if (cmd.equals("OK")) { //Process the password.
            chooseOK = true;
            this.setVisible(false);
        }
    }
}
 