/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.shared;

import be.cardon.utils.X500Parser;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.security.KeyStore;
import java.security.KeyStoreException;
import javax.security.auth.x500.X500Principal;
import java.text.SimpleDateFormat;
import java.security.Provider;

/**
 *
 * @author Rodolphe
 */
public class KeyStoreEntry {
    
    private KeyStore loadedStore;
    private String alias;
    
    private String issuer;         //cache
    private String issuerCN;       //cache
    private String subject;        //cache
    private String subjectCN;      //cache
    private String expirationDate; //cache
    
    /**
     * Creates a new instance of KeyStoreEntry 
     */
    public KeyStoreEntry(KeyStore loadedStore, String alias) 
    throws KeyStoreException, CertificateException{
        this.loadedStore = loadedStore;
        Certificate cert = loadedStore.getCertificate(alias);
        if(!(cert instanceof X509Certificate)){
            throw new java.security.cert.CertificateException("Certificate entry should be an X509Certificate");
        }
        
        X509Certificate x509cert = (X509Certificate) cert;
        this.alias = alias;
        X500Principal subjPrinc = x509cert.getSubjectX500Principal();
        X500Principal issuerPrinc = x509cert.getIssuerX500Principal();
        this.issuer = issuerPrinc.toString();
        this.issuerCN = new X500Parser(issuerPrinc).getCN();
        this.subject = subjPrinc.toString();
        this.subjectCN = new X500Parser(subjPrinc).getCN();
        this.expirationDate = new SimpleDateFormat().format(x509cert.getNotAfter());
    }
    
    public String getAlias(){return this.alias;}
    public String getIssuer(){return this.issuer;}
    public String getIssuerCN(){return this.issuerCN;}
    public String getSubject(){return this.subject;}
    public String getSubjectCN(){return this.subjectCN;}
    public String getExpirationDate(){return this.expirationDate;}
    
    public Provider getProvider(){
        return this.loadedStore.getProvider();
    }
    public KeyStore getLoadedKeyStore(){
        return this.loadedStore;
    }
    public Certificate[] getCertificateChain() throws java.security.KeyStoreException{
        return this.loadedStore.getCertificateChain(this.alias);
    }
    
}
