/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.shared;

import be.cardon.utils.Debug;
import java.security.KeyStore;
import java.security.cert.Certificate;
import javax.swing.*;
import javax.swing.table.*;
import java.util.List;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.Dimension;
import java.util.Enumeration;

/**Table that contains the certificates of a keystore
 *
 * @author Rodolphe
 */
public class KeyStoreTable extends JTable implements ListSelectionListener{
    
    protected List<KeyStoreEntry>  tableEntries;
    private KeyStoreEntry selectedKSEntry;
    private KeyStore ks;
    
    /** Creates a new instance of KeyStoreTable, loaded keystore. */
    public KeyStoreTable(KeyStore ks) throws java.security.KeyStoreException{
        super();
        
        this.ks = ks;
        this.tableEntries = KeyStoreUtils.getTableEntries(this.ks);
        
        ListSelectionModel rowSM = this.getSelectionModel();
        rowSM.addListSelectionListener(this);
        this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.setModel(this.tableModel);
        this.getColumnModel().getColumn(0).setPreferredWidth(200);
        this.getColumnModel().getColumn(1).setPreferredWidth(200);
        this.getColumnModel().getColumn(2).setPreferredWidth(100);
        this.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        this.setPreferredScrollableViewportSize(new Dimension(550,100));
        //this.setMaximumSize(new Dimension(250,100));
        //Utils.setMaxSizeUnfinite(thisthis);
        
        
    }
    
    private final static String[] columnNames =
    {"Subject", "Issuer", "Expiration date"};
    
    private AbstractTableModel tableModel = new AbstractTableModel() {
        public String getColumnName(int col) {
            return columnNames[col].toString();
        }
        
        public int getRowCount() {
            return tableEntries.size();
        }
        
        public int getColumnCount() {
            return columnNames.length;
        }
        public Object getValueAt(int row, int col) {
            KeyStoreEntry entry = tableEntries.get(row);
            switch(col){
                case 0: return entry.getSubjectCN();
                case 1: return entry.getIssuerCN();
                case 2: return entry.getExpirationDate();
            }
            return null;
        }
        public boolean isCellEditable(int row, int col) {
            return false;
        }
    };
    
    //ListSelectionListener interface
    public void valueChanged(ListSelectionEvent e) {
        this.repaint(); // bug resolved
        
        //The getValueIsAdjusting method returns true if the user is
        //still manipulating the selection.
        if (e.getValueIsAdjusting()) return;
        ListSelectionModel lsm =
                (ListSelectionModel)e.getSource();
        if (lsm.isSelectionEmpty()) {
            this.selectedKSEntry = null;
        } else {
            KeyStoreEntry kse = tableEntries.get(lsm.getMinSelectionIndex());
            this.selectedKSEntry = kse;
            
        }
    }
    
    /**return null if no key is selected.*/
    public KeyStoreEntry getSelectedEntry(){
        if(this.selectedKSEntry==null){
            return null;
        }
        return this.selectedKSEntry;
    }
}

