/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
*/

package be.cardon.sign.standalone;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.io.File;
import java.util.Iterator;
import java.util.ArrayList;
import be.cardon.utils.Debug;
import be.cardon.sign.*;
import be.cardon.sign.shared.GUIUtils;
import be.cardon.sign.shared.DialogBox;


/**
 *
 * @author Rodolphe
 */
public class FilesPanel extends JPanel implements ActionListener{
    
    private XMLSignerGUI parentFrame;
    private JPanel inputPanel;
    private JPanel outputPanel;
    private JTable inputFilesTable;
    private JButton addButton;
    private JButton removeButton;
    private JButton propertiesButton;
    private JButton openButton;
    private JTextField outputSignatureField;
    private JButton browseOutputButton;
    private JPanel tablePanel;
    private JButton previousButton;
    private JButton nextButton;
    
    public java.util.List<FileItem> inputFileItems;
    
    /**
     * Creates a new instance of FilesPanel 
     */
    public FilesPanel(XMLSignerGUI parentFrame){
        super();
        this.parentFrame = parentFrame;
        if(parentFrame.getXMLSigner()!=null){
            this.inputFileItems = parentFrame.getXMLSigner().getInputFileItems();
        }else{
            System.out.println("DEBUG : FilesPanel : Debug XMLSigner is null");
            //for testing only
            this.inputFileItems = new ArrayList<FileItem>();
        }
        
        JTextArea descriptionArea = GUIUtils.newTextAreaNote(this, "Add files to sign.");
        inputPanel = new JPanel();
        inputPanel.setBorder(BorderFactory.createTitledBorder("Files to sign"));
        inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.Y_AXIS));
        JScrollPane scroll = new JScrollPane(inputFilesTable());
        tablePanel = new JPanel();
        tablePanel.add(scroll);
        //addBorder(inputTable);
        inputPanel.add(tablePanel);
        
        
        JPanel InputButtonPanel = new JPanel();
        InputButtonPanel.setLayout(new BoxLayout(InputButtonPanel, BoxLayout.X_AXIS));
        addButton = new JButton("Add files...");
        addButton.addActionListener(this);
        removeButton = new JButton("Remove from list");
        removeButton.addActionListener(this);
        propertiesButton = new JButton("View Properties");
        propertiesButton.addActionListener(this);
        openButton = new JButton("Open file");
        openButton.addActionListener(this);
        openButton.setEnabled(false);
        removeButton.setEnabled(false);
        propertiesButton.setEnabled(false);
        InputButtonPanel.add(addButton);
        InputButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        InputButtonPanel.add(removeButton);
        InputButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        InputButtonPanel.add(openButton);
        
        //InputButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        InputButtonPanel.add(Box.createHorizontalGlue());
        inputPanel.add(InputButtonPanel);
    
        outputPanel = new JPanel();
        outputPanel.setBorder(BorderFactory.createTitledBorder("Output signature file"));
        //outputPanel.setLayout(new BoxLayout(outputPanel, BoxLayout.LINE_AXIS));
        outputSignatureField = new JTextField();
        outputSignatureField.setPreferredSize(new Dimension(300,20));
        
        browseOutputButton = new JButton("Browse..."); //open JFileChooser
        browseOutputButton.addActionListener(this);
        outputPanel.add(outputSignatureField);
        outputPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        outputPanel.add(browseOutputButton);
        
        JTextArea noteArea = GUIUtils.newTextAreaNote(this, "Note : The signature will contains a relative path" +
                " to the signed documents. \nStore by preference the signature in the" +
                " same directory.");
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        this.add(descriptionArea);
        this.add(Box.createRigidArea(new Dimension(0, 5)));
        this.add(inputPanel);
        this.add(outputPanel);
        this.add(Box.createRigidArea(new Dimension(0, 5)));
        this.add(noteArea);
        noteArea.setPreferredSize(new Dimension(0,50));
        //DEBUG
        //addBorder(descriptionArea);
        //addBorder(inputPanel);
        //addBorder(this);
        
        //parent elements
        this.previousButton = parentFrame.previousButton;
        this.nextButton = parentFrame.nextButton;
    }
    
    /* now in utils
    public static void addBorder(JComponent comp){
        comp.setBorder(BorderFactory.createLineBorder(Color.BLACK));
    }
     */  
    
    private JTable inputFilesTable(){

        
        inputFilesTable = new JTable();
        
        //Ask to be notified of selection changes.
        ListSelectionModel rowSM = inputFilesTable.getSelectionModel();
        rowSM.addListSelectionListener(this.listSelectListener);
        
        inputFilesTable.setModel(tableModel);
        inputFilesTable.getColumnModel().getColumn(0).setPreferredWidth(100);
        inputFilesTable.getColumnModel().getColumn(1).setPreferredWidth(75);
        inputFilesTable.getColumnModel().getColumn(2).setPreferredWidth(225);
        inputFilesTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        inputFilesTable.setPreferredScrollableViewportSize(new Dimension(400,200));
        
        inputFilesTable.setMaximumSize(new Dimension(500,500));
        return inputFilesTable;
    }
    
    private ListSelectionListener listSelectListener = new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
            //Ignore extra messages.
            if (e.getValueIsAdjusting()) return;

            ListSelectionModel lsm =
                    (ListSelectionModel)e.getSource();
            if (lsm.isSelectionEmpty()) {
                removeButton.setEnabled(false);
                propertiesButton.setEnabled(false);
                openButton.setEnabled(false);
            } else {
                int selectedRow = lsm.getMinSelectionIndex();
                removeButton.setEnabled(true);
                propertiesButton.setEnabled(true);
                if(lsm.getMinSelectionIndex()!=lsm.getMaxSelectionIndex()){
                    openButton.setEnabled(false);
                }else{
                    //only one item selected
                    openButton.setEnabled(true);
                }

            }
        }
    };
 
    
    private final String[] columnNames = {"Filename",   "Size",  "Path"};
    private AbstractTableModel tableModel = new AbstractTableModel() {
            public String getColumnName(int col) {
                return columnNames[col].toString();
            }
            public int getRowCount() { 
                return inputFileItems.size(); }
            public int getColumnCount() { return columnNames.length; }
            public Object getValueAt(int row, int col) {
                FileItem fi = inputFileItems.get(row);
                switch(col){
                    case 0: return fi.getName();
                    case 1: return fi.getFriendlySize();
                    case 2: return fi.getFullPath();
                }
                return null;
            }
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
    

    
    /**Return the files or null if user cancelled the operation.*/
    public File[] AddfileChooser(){
        JFileChooser jfc = new JFileChooser();
        jfc.setDialogTitle("Add a file to sign");
        jfc.setDialogType(JFileChooser.OPEN_DIALOG);
        jfc.setMultiSelectionEnabled(true);
        //TO DO : extension filter
        int status = jfc.showDialog(parentFrame, "Add files");
        if(status == jfc.APPROVE_OPTION){
            return jfc.getSelectedFiles();
        }else{
            return null;
        }
    }
    
    /**Return the file or null if user cancelled the operation.*/
    public File SaveSignatureFileChooser(){
        JFileChooser jfc = new JFileChooser();
        jfc.setDialogTitle("Save signature to");
        jfc.setDialogType(JFileChooser.SAVE_DIALOG);
        jfc.setMultiSelectionEnabled(false);
        int status = jfc.showDialog(parentFrame, "OK");
        if(status == jfc.APPROVE_OPTION){
            return jfc.getSelectedFile();
        }else{
            return null;
        }
    }    
    
    public void actionPerformed(ActionEvent e) {
        try{
        Debug.println("clic!");
        if(parentFrame.getState()!=XMLSigner.STATE_SELECT_FILES){
            System.err.println("Warning FilePanel.actionPerformed() : state isn't STATE_SELECT_FILES");
            return;
        }
        
        if(e.getSource() == addButton){
            File[] files = AddfileChooser();
            if(files==null){return;} //user cancel
            for(int index = 0; index < files.length; index ++){
                addFileToList(files[index]);
            }
        }else if(e.getSource() == removeButton){
            ListSelectionModel lsm = inputFilesTable.getSelectionModel();
            if(lsm.isSelectionEmpty()){
                return;
            }else{
                for(int i=lsm.getMinSelectionIndex();i<=lsm.getMaxSelectionIndex();i++){
                    if(lsm.isSelectedIndex(i)){
                        this.removeFileFromList(i);
                    }else{
                    }
                }
            }
        }else if(e.getSource() == openButton){
            ListSelectionModel lsm = inputFilesTable.getSelectionModel();
            if(lsm.isSelectionEmpty()){
                return;
            }else{
                int i=lsm.getMinSelectionIndex();
                FileItem fi = inputFileItems.get(i);
                File file = new File(fi.getFullPath());
                if(!be.cardon.utils.OperatingSystem.open(file)){
                    DialogBox.showWarningBox(parentFrame, "Cannot open file!");
                }
            }
        }else if(e.getSource() == browseOutputButton){
            File signatureFile = SaveSignatureFileChooser();
            if(signatureFile==null){return;}
            outputSignatureField.setText(signatureFile.getAbsolutePath());
            
        }else if(e.getSource() == this.nextButton){
            //check that input files list is not empty
            if(inputFileItems.size()==0){
                DialogBox.showWarningBox(this.parentFrame, "Please add files to sign !");
                return;
            }
            
            //check that signature file field is valid
            String signatureFileString = outputSignatureField.getText();
            if(signatureFileString.length()==0){
                DialogBox.showWarningBox(this.parentFrame, "Please specify the signature output file.");
                return;
            }
            File signatureFile = validFileString(signatureFileString);
            if(signatureFile==null){
                DialogBox.showWarningBox(this.parentFrame, "Invalid signature output file.");
                return;
            }
            if(signatureFile.exists()){
                DialogBox.showWarningBox(this.parentFrame, "Signature output file already exists." +
                        "Please select another file.");
                return;                
            }
            this.parentFrame.getXMLSigner().setSignatureFile(signatureFile);
            
            //display the next panel
            this.parentFrame.showNextPanel();
            
        }else{
            System.err.println("FilesPanel: ActionPerformed: Action undefined for source "+e.toString());
        }
        }catch(Exception e2){
            DialogBox.showExceptionBox(this.parentFrame, e2);
        }
    }
    
    /**return null if the file string is invalid.*/
    public static File validFileString(String path){
        try{
            if(path==null){return null;}
            if(path.length()==0){return null;}
            File file = new File(path).getAbsoluteFile();
            return file;
        }catch(Exception e){
            return null;
        }
    }
    
    /**add a file in the list if not already in the list.*/
    public void addFileToList(File f){
        for(Iterator<FileItem> it = inputFileItems.iterator();it.hasNext();){
            if(f.equals(it.next().file)){
                return;
            }
        }
        
        inputFileItems.add(new FileItem(f));
        int index = inputFileItems.size()-1;
        //prevenir la table du changement
        tableModel.fireTableRowsInserted(index,index);
    }
    /**remove the file item from table.*/
    public void removeFileFromList(int index){
        inputFileItems.remove(index);
        tableModel.fireTableRowsDeleted(index,index);
    }
    
    /**For debugging only.*/
    /*
    public static void main(String[] args) {
        XMLSignerGUI frame = new XMLSignerGUI(null);
        frame.showMainPanel(XMLSigner.STATE_SELECT_FILES);
    }
     **/
}
