/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.standalone;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.security.KeyStore;
import be.cardon.utils.Debug;
import java.io.File;
import be.cardon.sign.shared.KeyStoreTable;
import be.cardon.sign.shared.GUIUtils;
import be.cardon.sign.shared.TreeEntry;
import be.cardon.sign.shared.KeyStoreEntry;
import be.cardon.sign.*;
import be.cardon.sign.shared.DialogBox;



/**
 *
 * @author Rodolphe
 */
public class KeyPanel extends JPanel
        implements TreeSelectionListener, ActionListener{
    
    private XMLSignerGUI parentFrame;
    private JTree providerTree;
    private Box providerRightPanel;
    private KeyStoreTable keyStoreTable; //current visible table
    private final static String MS_CRYPTO_ID = "mscrypto";
    private final static String JKS_ID = "jks";
    private final static String PKCS12_ID = "pkcs12";
    private final static String PKCS11_ID = "pkcs11";
    
    /**
     * Creates a new instance of KeyPanel
     */
    public KeyPanel(XMLSignerGUI parentFrame){
        super();
        
        this.parentFrame = parentFrame;
        if(parentFrame==null){
            Debug.println("WARNING : KeyPanel: parentFrame=null");
        }
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        
        JLabel selectKeyLabel = new JLabel("Select your private key.");
        selectKeyLabel.setAlignmentX(0.0f);
        
        // tree
        JTree providerTree = providerTree();
        this.providerTree = providerTree;
        GUIUtils.setMaxSizeUnfinite(this.providerTree);
        
        //scroll with tree
        JScrollPane providerScroll = new JScrollPane(providerTree);
        //providerScroll.setPreferredSize(new Dimension(300,300));
        
        this.providerRightPanel = Box.createVerticalBox();
        this.providerRightPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        
        
        
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, providerScroll, this.providerRightPanel);
        splitPane.setPreferredSize(new Dimension(700,400));
        splitPane.setBorder(BorderFactory.createTitledBorder("Private Key"));
        splitPane.setAlignmentX(0.0f);
        GUIUtils.setMaxSizeUnfinite(splitPane);
        
        
        
        this.add(selectKeyLabel);
        this.add(splitPane);
        
        this.setAlignmentX(0.0F);
        
        //displayRootPanel();
        displayRootPanel();
    }
    
    private JTree providerTree(){
        
        DefaultMutableTreeNode top =
                new DefaultMutableTreeNode(new TreeEntry("Providers", "root"));
        if(be.cardon.utils.OperatingSystem.isWindows()){
            top.add(new DefaultMutableTreeNode(new TreeEntry("Windows store", MS_CRYPTO_ID)));
        }
        top.add(new DefaultMutableTreeNode(new TreeEntry("Java key store", JKS_ID)));
        top.add(new DefaultMutableTreeNode(new TreeEntry("PKCS#12 File", PKCS12_ID)));
        top.add(new DefaultMutableTreeNode(new TreeEntry("PKCS#11 DLL library", PKCS11_ID)));
        
        JTree provTree = new JTree(top);
        provTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        provTree.addTreeSelectionListener(this);
        
        return provTree;
        
    }
    
    public void valueChanged(TreeSelectionEvent e) {
        Debug.println("KeyPanel : valueChanged (TreeSelectionEvent)");
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
        this.providerTree.getLastSelectedPathComponent();
        
        if (node == null) return;
        
        Object nodeInfo = node.getUserObject();
        if (node.getUserObject() instanceof TreeEntry) {
            Debug.println("KeyPanel : valueChanged, clic on TreeEntry");
            this.keyStoreTable = null; //reset the pointer to keyStoreTable
            TreeEntry treeEntry = (TreeEntry)node.getUserObject();
            String id = treeEntry.id;
            if(id=="root")      {displayRootPanel(); return; }
            if(id==MS_CRYPTO_ID){displayMSCryptoPanel(); return;}
            if(id==this.JKS_ID){displayJKSPanel(); return;}
            if(id==this.PKCS11_ID){//displayPKCS11Panel();
                return;
            }
            if(id==this.PKCS12_ID){//displaPKCS12Panel();
                return;
            }
        } else {
            //nothing
        }
    }
    
    public void actionPerformed(ActionEvent e){
        Debug.println("KeyPanel : action performed !");
        Object source = e.getSource();
        if(source == this.parentFrame.nextButton){
            
            //check if a keyStoreEntry is selected
            if(this.keyStoreTable!=null){
                KeyStoreEntry kse = this.keyStoreTable.getSelectedEntry();
                if(kse!=null){
                    this.parentFrame.getXMLSigner().setSigningKeyStoreEntry(kse);
                    this.parentFrame.showNextPanel();
                    return;
                }
            }
            DialogBox.showWarningBox(this.parentFrame, "Please select a private key.");
            return;
            
        }else if(source == this.parentFrame.previousButton){
            this.parentFrame.showPreviousPanel();
        }
        
    }
    
    /**when Root node is selected.*/
    private void displayRootPanel(){
        JTextArea descr = GUIUtils.newTextAreaNote(this,"Select a provider.");
        providerRightPanel.removeAll();
        providerRightPanel.add(descr);
        providerRightPanel.revalidate();
        providerRightPanel.repaint();
    }
    private void displayMSCryptoPanel(){
        Debug.println("KeyPanel: displayMSCryptoPanel");
        try{
            JLabel description = new JLabel("Content of 'My' store.");
            description.setAlignmentX(0.0f);
            
            KeyStore keystore = XMLSigner.getMSKeyStore();
            KeyStoreTable keyStoreTable = new KeyStoreTable(keystore);
            this.keyStoreTable = keyStoreTable;
            JScrollPane scroll= new JScrollPane(keyStoreTable);
            scroll.setAlignmentX(0.0f);
            providerRightPanel.removeAll();
            providerRightPanel.add(description);
            providerRightPanel.add(scroll);
            revalidateAndRepaint(providerRightPanel);
        }catch(Exception e){
            DialogBox.showExceptionBox(this.parentFrame, e);
            providerRightPanel.removeAll();
            providerRightPanel.add(new JLabel("Unable to access the keyStore."));
        }
    }
    private void displayJKSPanel(){
        JKSPanel jksPanel= new JKSPanel();
        providerRightPanel.removeAll();
        providerRightPanel.add(jksPanel);
        revalidateAndRepaint(providerRightPanel);
        
    }
    
    // ********************** JKS PANEL *******************************
    
    class JKSPanel extends JPanel implements ActionListener{
        
        private JButton defaultFileButton;
        private JButton selectFileButton;
        private JLabel filePath;
        private KeyStoreTable ksTable;
        private Box ksBox; //will contained the scrollPane with the list
        
        public JKSPanel(){
            Debug.println("KeyPanel: displayJKSPanel");
            this.defaultFileButton = new JButton("Default JKS");
            defaultFileButton.addActionListener(this);
            this.selectFileButton = new JButton("Select JKS File...");
            selectFileButton.addActionListener(this);
            
            Box buttonsBox = Box.createHorizontalBox();
            buttonsBox.add(defaultFileButton);
            buttonsBox.add(selectFileButton);
            buttonsBox.add(Box.createHorizontalGlue());
            
            filePath = new JLabel("No file selected.");
            
            Box fileBox = Box.createHorizontalBox();
            fileBox.add(filePath);
            fileBox.add(Box.createHorizontalGlue());
            
            this.ksBox = Box.createHorizontalBox();
            
            this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
            this.add(buttonsBox);
            this.add(fileBox);
            this.add(this.ksBox);
            
            
        }
        
        public void actionPerformed(ActionEvent e){
            if(e.getSource()==defaultFileButton){
                try{
                File JKSFile = XMLSigner.getDefaultJKSKeyStore();
                if(JKSFile == null){
                    DialogBox.showWarningBox(parentFrame, "Unable to determines the default Java KeyStore");
                    return;
                }
                this.openJKSFileStore(JKSFile);
                }catch(XMLSignerException ex){
                    DialogBox.showExceptionBox(parentFrame, ex);
                    return;
                }
            }
            if(e.getSource()==selectFileButton){
                File JKSFile = this.JKSfileChooser();
                if(JKSFile==null){return;} //user cancel
                openJKSFileStore(JKSFile);
            }
        }
        public File JKSfileChooser(){
            JFileChooser jfc = new JFileChooser();
            jfc.setDialogTitle("Select a JKS store");
            jfc.setDialogType(JFileChooser.OPEN_DIALOG);
            jfc.setMultiSelectionEnabled(false);
            //TO DO : extension filter
            int status = jfc.showDialog(parentFrame, "Open JKS store");
            if(status == jfc.APPROVE_OPTION){
                return jfc.getSelectedFile();
            }else{
                return null;
            }
        }
        
        public void openJKSFileStore(File JKSFile){
            try{
                if(JKSFile==null){
                    Debug.println("KeyPane: openJKSFileStore: JKSfile is null");
                    return;
                }
                KeyStore ks=null;
                //attemp to open the store without password.
                ks = XMLSigner.getJKSStore(JKSFile, null);
                if(ks==null){
                    Debug.println("KeyPanel: wrong pass 1");
                    //open dialog
                    boolean passUnvalid = true;
                    boolean firstAsk = true;
                    while(ks==null){
                        char[] pass = DialogBox.askPassword(
                                parentFrame, "Enter the keystore password.",firstAsk);
                        if(pass==null){return;} //user cancelled
                        firstAsk = false;
                        ks = XMLSigner.getJKSStore(JKSFile, pass);
                    }
                }
                if(ks==null){
                    Debug.println("keystore : null pointer");
                    return;
                }
                //here valid password
                KeyStoreTable ksTable = new KeyStoreTable(ks);
                keyStoreTable = ksTable;
                ksBox.removeAll();
                ksBox.add(new JScrollPane(ksTable));
                filePath.setText("File: "+JKSFile.getAbsolutePath());
                ksBox.revalidate();
                ksBox.repaint();
                
            }catch(Exception xmlEx){
                DialogBox.showExceptionBox(parentFrame, xmlEx);
                ksBox.removeAll();
                ksBox.add(new JLabel("Unable to access the JKS keyStore."));
            }
        }
    };
    
    
    // **********************                *******************************
    
    
    /** revalidate + repaint component.*/
    private void revalidateAndRepaint(JComponent comp){
        comp.revalidate();
        comp.repaint();
    }
    
    
    
}
