/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.standalone;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.Writer;
import java.security.PrivateKey;
import be.cardon.sign.*;
import be.cardon.sign.shared.DialogBox;


/**
 *
 * @author Rodolphe
 */
public class SignPanel extends JPanel implements ActionListener{
    
    private XMLSignerGUI xmlSignerGUI;
    private JTextArea summary;
    private JTextArea log;
    private JScrollPane summaryScrollPane;
    private JScrollPane logWindowScrollPane;
    private JButton signButton;
    
    /**
     * Creates a new instance of SignPanel
     */
    public SignPanel(XMLSignerGUI xmlsignerGUI) {
        super();
        this.xmlSignerGUI = xmlsignerGUI;
        
        summary = new JTextArea();
        summary.setBackground(Color.WHITE);
        summary.setEditable(false);
        summary.setOpaque(true);
        
        summaryScrollPane = new JScrollPane(summary);
        summaryScrollPane.setOpaque(true);
        summaryScrollPane.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Summary"),
                BorderFactory.createLoweredBevelBorder()));
        summaryScrollPane.setPreferredSize(new Dimension(600, 300));
        summaryScrollPane.getVerticalScrollBar().setAutoscrolls(false);
        summaryScrollPane.getHorizontalScrollBar().setAutoscrolls(false);
        
        signButton = new JButton("Sign !");
        signButton.addActionListener(this);
        Box signBox = Box.createHorizontalBox();
        signBox.add(signButton);
        signBox.add(Box.createHorizontalGlue());
        
        log = new JTextArea();
        //log.setBackground(Color.WHITE);
        log.setEditable(false);
        log.setOpaque(false);

        
        logWindowScrollPane = new JScrollPane(log);
        logWindowScrollPane.setPreferredSize(new Dimension(600, 100));
        
        logWindowScrollPane.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Log window"),
                BorderFactory.createLoweredBevelBorder()));
        logWindowScrollPane.getHorizontalScrollBar().setAutoscrolls(false);
        
        this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
        this.add(summaryScrollPane);
        this.add(signBox);
        this.add(logWindowScrollPane);
        
        //logTextArea.setEditable(false);
    }
    
    
    public void setVisible(boolean bool){
        super.setVisible(bool);
        if(bool==true){
            this.refreshSignPanel();
        }
    }
    
    public void refreshSignPanel(){
        this.clearLog();
        this.clearSummary();
        this.generateSummary();
        summaryScrollPane.getHorizontalScrollBar().setValue(
            summaryScrollPane.getHorizontalScrollBar().getMinimum());
        summaryScrollPane.getVerticalScrollBar().setValue(
            summaryScrollPane.getVerticalScrollBar().getMinimum());
        
    }
    
    public void generateSummary(){
        Writer summaryWriter = this.summaryWriter;
        xmlSignerGUI.getXMLSigner().summary(summaryWriter);
    }
    
    public void sign() throws XMLSignerException{
        Writer writer = this.logWriter;
        xmlSignerGUI.getXMLSigner().sign(writer);
    }
    
    public void clearSummary(){
        summary.setText("");
    }
    public void printLineSummary(String text){
        printSummary(text + "\n");

    }
    public void printSummary(String text){
        summary.setText(summary.getText() + text);
    }
    public void clearLog(){
        log.setText("");
    }
    public void printLineLog(String text){
        printLog(text + "\n");
    }
    public void printLog(String text){
        log.setText(log.getText() + text);
        //autoscroll
        int max=logWindowScrollPane.getVerticalScrollBar().getMaximum();
        logWindowScrollPane.getVerticalScrollBar().setValue(max);
        logWindowScrollPane.revalidate();
    }
    
    /** retrives private key. Ask for password if needed. return null if user cancelled
     * the operation
     */
    private PrivateKey getPrivateKey() throws XMLSignerException{
        XMLSigner xmlSigner = this.xmlSignerGUI.getXMLSigner();
        PrivateKey key = null;
        key = xmlSigner.getPrivateKey(null);
        boolean firstAsk = true;
        while(key==null){
            char[] password = DialogBox.askPassword(xmlSignerGUI, "Enter the key password", firstAsk);
            firstAsk = false;
            if(password == null){return null;} //user cancel
            key = xmlSigner.getPrivateKey(password);
        }
        return key;
    }
    
    public void actionPerformed(ActionEvent event){
        Object source = event.getSource();
        if(source==signButton){
            try{
            PrivateKey privKey = this.getPrivateKey();
            if(privKey==null){return;} //user cancel
            this.xmlSignerGUI.getXMLSigner().setSigningPrivateKey(privKey);
            this.sign();
            DialogBox.showWarningBox(xmlSignerGUI, "Signature successfull");
            this.signButton.setEnabled(false);
            xmlSignerGUI.previousButton.setEnabled(false);
            }catch(XMLSignerException e){
                e.printStackTrace();
                DialogBox.showExceptionBox(xmlSignerGUI, e);
                return;
            }
       }
        
        if(source==this.xmlSignerGUI.previousButton){
            this.xmlSignerGUI.showPreviousPanel();
        }
    }
    
    private Writer summaryWriter = new Writer(){
        /**Close the stream, flushing it first. */
        public void close(){ }
        /**Flush the stream. */
        public void flush(){ }
        /**Write a portion of an array of characters. */
        public void write(char[] cbuf, int off, int len) {
            String text = String.copyValueOf(cbuf, off, len);
            printSummary(text);
        }
    };
    
    private Writer logWriter = new Writer(){
        /**Close the stream, flushing it first. */
        public void close(){ }
        /**Flush the stream. */
        public void flush(){ }
        /**Write a portion of an array of characters. */
        public void write(char[] cbuf, int off, int len) {
            String text = String.copyValueOf(cbuf, off, len);
            printLog(text);
        }
    };
    
}
