/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.standalone;

import be.cardon.utils.Debug;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import be.cardon.sign.*;

/**
 *
 * @author Rodolphe
 */
public class XMLSignerGUI extends JFrame{
    
    public JPanel mainPane;
    public JButton previousButton, nextButton;
    private XMLSigner xmlSigner;
    private JPanel framePane;
    private JPanel previousNextPane;
    private FilesPanel fileSelectionPanel;
    private KeyPanel keySelectionPanel;
    private SignPanel signPanel;
    private JPanel currentPanel;
    
    
    /**Constructor.*/
    public XMLSignerGUI(XMLSigner xmlSigner) {
        super("XML Signer");
        
        if(xmlSigner==null){
            System.out.println("DEBUG : XMLSignerGUI : XMLSigner is null");
        }else{
            this.xmlSigner = xmlSigner;
        }
        //look and feel
        try{
        UIManager.setLookAndFeel(
            //UIManager.getCrossPlatformLookAndFeelClassName());
            UIManager.getSystemLookAndFeelClassName());
        }catch(Exception e){
            System.err.println("Look And Feel failed.");
        }

        this.makemenu();
        //this.setMenuBar(...)
        
        
        
        framePane = new JPanel();
        framePane.setLayout(new BoxLayout(framePane, BoxLayout.Y_AXIS));
        
        mainPane = new JPanel();
        mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.Y_AXIS));
        mainPane.setAlignmentX(0.5f);
        //mainPane.setMaximumSize(new Dimension(500,300));
        
        previousNextPane = new JPanel();
        previousNextPane.setLayout(new BoxLayout(previousNextPane, BoxLayout.X_AXIS));
        previousButton = new JButton("Previous");
        nextButton = new JButton("Next");
        previousNextPane.add(Box.createHorizontalGlue());
        previousNextPane.add(previousButton);
        previousNextPane.add(Box.createRigidArea(new Dimension(5, 0)));
        previousNextPane.add(nextButton);
        
        framePane.add(mainPane);
        framePane.add(previousNextPane);
        framePane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        this.setContentPane(framePane); 
        
        //init Panels
        this.fileSelectionPanel = new FilesPanel(this);
        this.fileSelectionPanel.setVisible(false);
        this.keySelectionPanel = new KeyPanel(this);
        this.keySelectionPanel.setVisible(false);
        this.signPanel = new SignPanel(this);
        this.signPanel.setVisible(false);

        //this.setResizable(false);
        this.setLocationByPlatform(true);
        this.setVisible(true);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        if(xmlSigner==null){
            //the main method of XMLSigner was called
            //for testing only
            this.showMainPanel(xmlSigner.STATE_SELECT_FILES);
        }else{
            this.showMainPanel(xmlSigner.getState());
        }
        //this.pack();
        
        
    }
    
    public XMLSigner getXMLSigner(){
        return this.xmlSigner;
    }
    
    public void showNextPanel(){
        int newState = this.getXMLSigner().getNextState();
        this.getXMLSigner().setState(newState);
        showMainPanel();
    }
    
    public void showPreviousPanel(){
        int newState = this.getXMLSigner().getPreviousState();
        this.getXMLSigner().setState(newState);
        showMainPanel();
    }
    public void showMainPanel(){
        int currentState = this.xmlSigner.getState();
        Debug.println("XMLSignerGUI: showMainPanel: currentState="+currentState);
        showMainPanel(currentState);
    }
    
    public void showMainPanel(int XMLSignerStatus){
        this.mainPane.removeAll();
        this.previousButton.setEnabled(true); //default value
        this.nextButton.setEnabled(true);     //default value
        JButton[] progressButtons = {this.previousButton, this.nextButton};
        if(currentPanel!=null){
            currentPanel.setVisible(false);
        }
        if(XMLSignerStatus == XMLSigner.STATE_SELECT_FILES){
                currentPanel = this.fileSelectionPanel;
                ActionListener listener = this.fileSelectionPanel; 
                currentPanel.setVisible(true);
                this.mainPane.add(currentPanel);
                this.previousButton.setEnabled(false);
                setAloneListener(progressButtons, listener);
        }else if(XMLSignerStatus == XMLSigner.STATE_SELECT_KEY){
                currentPanel = this.keySelectionPanel;
                ActionListener listener = this.keySelectionPanel;
                currentPanel.setVisible(true);
                this.mainPane.add(currentPanel);
                setAloneListener(progressButtons, listener);
        }else if(XMLSignerStatus == XMLSigner.STATE_SIGNING){
                currentPanel = this.signPanel;
                ActionListener listener = this.signPanel;
                currentPanel.setVisible(true);
                this.mainPane.add(currentPanel);
                this.nextButton.setEnabled(false);
                setAloneListener(progressButtons, listener);
        }
        this.pack();
        //this.getContentPane().repaint();
        
    }
    
    /**Removes all the ActionListeners of a array of button, and add the given
     * listener.  */
    private void setAloneListener(JButton[] buttons, ActionListener listener){
        for(int i=0;i<buttons.length;i++){
            setAloneListener(buttons[i], listener);
        }
    }
    
    /**Removes all the ActionListeners of a button, and add the given
     * listener.  */
    private void setAloneListener(JButton button, ActionListener listener){
        removeAllActionListeners(button);
        button.addActionListener(listener);
    }
    
    /**Removes all the ActionListeners of a button.*/
    private void removeAllActionListeners(JButton button){
        ActionListener[] al = button.getActionListeners();
        for(int i = 0; i < al.length; i++){
            button.removeActionListener(al[i]);
        }
    }
    
    private void makemenu(){
        //to do
    }
    
    
}
