/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.wizard;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import be.cardon.utils.Debug;
import be.cardon.sign.WebSignApplication;
import be.cardon.sign.shared.GUIUtils;

/**Panel with a blank panel (title and description) + separator + content panel
 * @author Rodolphe
 */
public abstract class WizardDefaultPanel extends WizardPane{
    
    protected WizardFrame wizardFrame;
    
    protected JPanel descriptionPanel;
    protected JPanel contentPanel;
    protected JLabel titleLabel;
    protected JTextArea descriptionTextArea;
    
    protected JButton helpButton;
    protected JButton previousButton;
    protected JButton nextButton;
    protected JButton cancelButton;
    
    
    private String cardName;
    
    /** Creates a new instance of WizardDefaultPanel */
    public WizardDefaultPanel(String cardName) {
        super();
        
        this.cardName = cardName;
        Font defaultFont = new JLabel().getFont();
        if(defaultFont==null){
            throw new NullPointerException("defaultFont");
        }
        titleLabel = new JLabel("Title of this panel");
        titleLabel.setFont(defaultFont.deriveFont(Font.BOLD));
        titleLabel.setOpaque(false);
        titleLabel.setAlignmentX(0.0f);
        descriptionTextArea = new JTextArea("Put here the description of your panel." +
                "This description can take more that one line. Like this description that is" +
                " very very very very long, blablabla...");
        descriptionTextArea.setFont(defaultFont);
        descriptionTextArea.setWrapStyleWord(true);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setOpaque(false);
        descriptionTextArea.setAlignmentX(0.0f);
        descriptionTextArea.setPreferredSize(new Dimension(Integer.MAX_VALUE,30));
        descriptionTextArea.setMaximumSize(new Dimension(Integer.MAX_VALUE,30));
        descriptionTextArea.setEnabled(false); // cannot select the content with the mouse.
        descriptionTextArea.setDisabledTextColor(Color.black);
        
        
        descriptionPanel = new JPanel();
        descriptionPanel.setLayout(new BoxLayout(descriptionPanel, BoxLayout.Y_AXIS));
        descriptionPanel.setBackground(Color.white);
        descriptionPanel.setOpaque(true);
        descriptionPanel.setBorder(BorderFactory.createEmptyBorder(15,20,5,20));
        descriptionPanel.add(titleLabel);
        descriptionPanel.add(Box.createRigidArea(new Dimension(1,5)));
        descriptionPanel.add(descriptionTextArea);
        descriptionPanel.setAlignmentX(0.0f);
        //descriptionPanel.setPreferredSize(new Dimension(1, 75));
        //descriptionPanel.setMaximumSize(new Dimension(Integer.MAX_VALUE, 75));
        
        
        contentPanel = new JPanel();
        contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
        contentPanel.setAlignmentX(0.0f);
        
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        this.add(descriptionPanel);
        JSeparator separator = new JSeparator(JSeparator.HORIZONTAL);
        separator.setAlignmentX(0.0f);
        separator.setMaximumSize(new Dimension(Integer.MAX_VALUE,3));
        separator.setPreferredSize(new Dimension(Integer.MAX_VALUE, 3));
        
        this.add(separator);
        this.add(contentPanel);
        
    }
    public void setTitle(String title){
        titleLabel.setText(title);
    }
    public void setDescription(String description){
        descriptionTextArea.setText(description);
    }
    public void setPreviousButton(JButton previousButton){
        this.previousButton = previousButton;
    }
    public void setNextButton(JButton nextButton){
        this.nextButton = nextButton;
    }
    public void setCancelButton(JButton cancelButton){
        this.cancelButton = cancelButton;
    }
    public void setHelpButton(JButton helpButton){
        this.helpButton=helpButton;
    }
    public String getCardName(){
        return this.cardName;
    }
    public void setWizardFrame(WizardFrame wizardFrame){
        this.wizardFrame = wizardFrame;
    }
    
    public WebSignApplication getApplication(){
        return WebSignApplication.getInstance();
    }
    
    public void reinit(){
        //do nothing by default
        //can be overriden.
    }
    
    public boolean showDefaultQuitDialog(){
        int choice = JOptionPane.showConfirmDialog(
                wizardFrame,"Are you sure you " +
                "want to quit ? ",
                "Quit Confirmation",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.WARNING_MESSAGE);
        return (choice==JOptionPane.OK_OPTION);
    }
    
     public static ImageIcon getImageIcon(String filename, String description){
        return GUIUtils.getImageIcon(filename, description);
    }
}

