/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.wizard;

import be.cardon.sign.shared.GUIUtils;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Map;
import java.util.HashMap;
import be.cardon.utils.Debug;
import java.util.List;
import java.net.URL;

/**Main frame with the buttons Previous, Next, Cancel, help
 *
 * @author Rodolphe
 */
public class WizardFrame extends JFrame {
    
    private JPanel wizardPanel;
    private CardLayout wizardCardLayout;
    private JPanel buttonsPanel;
    private JButton helpButton;
    private JButton previousButton;
    private JButton nextButton;
    private JButton cancelButton;
    private Map<String,WizardPane> wizardPanelsMap; //map cardName => wizardPane
    
    
    
    ////////////////////////////// INIT //////////////////////////////
    
    /** Creates a new instance of MainFrame */
    public WizardFrame() {
        super("WebSigner Applet");
        this.setIconImage(this.getImageIcon("images/mail.gif", "mail").getImage());
        
        //look and feel
        try{
        UIManager.setLookAndFeel(
            //UIManager.getCrossPlatformLookAndFeelClassName());
            UIManager.getSystemLookAndFeelClassName());
        }catch(Exception e){System.err.println("Look And Feel failed.");}
        this.setPreferredSize(new Dimension(600,500));
        this.setSize(new Dimension(600, 400));
        //this.setResizable(false);
        this.setResizable(true);
        
        wizardCardLayout = new CardLayout();
        
        wizardPanel = new JPanel(wizardCardLayout);
        
        ImageIcon cancelIcon = this.getImageIcon("images/cancel.gif", "Cancel");
        ImageIcon helpIcon =  this.getImageIcon("images/help.gif", "Help");
        ImageIcon previousIcon =  this.getImageIcon("images/previous.gif", "Previous");
        ImageIcon nextIcon =  this.getImageIcon("images/next.gif", "Next");
        helpButton = new JButton("Help");
        helpButton.setIcon(helpIcon);
        previousButton = new JButton("Previous");
        previousButton.setIcon(previousIcon);
        nextButton = new JButton("Next");
        nextButton.setIcon(nextIcon);
        cancelButton = new JButton("Cancel");
        cancelButton.setIcon(cancelIcon);
        
        buttonsPanel = new JPanel();
        buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
        buttonsPanel.add(helpButton);
        buttonsPanel.add(Box.createHorizontalGlue());
        buttonsPanel.add(previousButton);
        buttonsPanel.add(Box.createRigidArea(new Dimension(5,1)));
        buttonsPanel.add(nextButton);
        buttonsPanel.add(Box.createRigidArea(new Dimension(20,1)));
        buttonsPanel.add(cancelButton);
        buttonsPanel.setBorder(BorderFactory.createEmptyBorder(10,5,10,5));
        
        
        Container contentPanel = this.getContentPane();
        contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
        contentPanel.add(wizardPanel);
        JSeparator separator = new JSeparator(JSeparator.HORIZONTAL);
        separator.setMaximumSize(new Dimension(Integer.MAX_VALUE,3));
        separator.setPreferredSize(new Dimension(Integer.MAX_VALUE, 3));
        
        contentPanel.add(separator);
        contentPanel.add(buttonsPanel);

        
        //initialize map
        wizardPanelsMap = new HashMap<String, WizardPane>();
        
        //add panels to wizardPanel...
        addToWizardPanel(new WizardPanelHello());
        addToWizardPanel(new WizardPanelTargetDir());
        addToWizardPanel(new WizardPanelDownload());
        addToWizardPanel(new WizardPanelViewFiles());
        addToWizardPanel(new WizardPanelSelectKey());
        addToWizardPanel(new WizardPanelSignAndUpload());
        //first panel to be displaid:
        this.setCurrentPanel(WizardPanelHello.cardName);
        this.pack();
    }
    
    //////////////////////////////////////////////////////////////////
    
    /* set the panel as the alone listener of the buttons, and display this pane.*/
    public void setCurrentPanel(String cardName){
        
            WizardPane panel = this.wizardPanelsMap.get(cardName);
            if(panel==null){
                System.err.println("WizardFrame: setCurrentPanel: unregistered card name: "+cardName);
                return;
            }
            JButton[] buttons = {helpButton, previousButton,
                                nextButton, cancelButton};
            for(int i=0;i<buttons.length;i++){
                JButton but = buttons[i];
                this.removeAllActionListeners(but);
                but.addActionListener(panel);
                but.setEnabled(true);
            }
            panel.reinit();
            wizardCardLayout.show(wizardPanel, panel.getCardName());
    }
    
    /**Removes all the ActionListeners of a button.*/
    private void removeAllActionListeners(JButton button){
        ActionListener[] al = button.getActionListeners();
        for(int i = 0; i < al.length; i++){
            button.removeActionListener(al[i]);
        }
    }
    
    /**add to wizard panel and set WizardFrame to panels.*/
    public void addToWizardPanel(WizardPane panel){
        this.wizardPanelsMap.put(panel.getCardName(), panel);
        this.wizardPanel.add(panel, panel.getCardName());
        panel.setWizardFrame(this);
        panel.setHelpButton(this.helpButton);
        panel.setPreviousButton(this.previousButton);
        panel.setNextButton(this.nextButton);
        panel.setCancelButton(this.cancelButton);
        
    }

    /**returns the image, or diplay a error in the console on error.*/
    public static ImageIcon getImageIcon(String filename, String description){
        return GUIUtils.getImageIcon(filename, description);
    }
}
