/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.wizard;

import be.cardon.utils.Debug;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.util.List;
import be.cardon.sign.shared.DialogBox;
import be.cardon.sign.communicator.Downloader;
import be.cardon.sign.communicator.DownloaderFile;
import java.util.Iterator;

/**
 *
 * @author Rodolphe
 */
public class WizardPanelDownload extends WizardDefaultPanel{
    
   private static final int timerRefreshTime = 100;
        
    public static String cardName = "downloadPanel";
    private JLabel globalStatus;
    private JProgressBar globalProgressBar;
    private JLabel currentFileStatus;
    private JLabel messageLabel;
    private JProgressBar currentFileProgressBar;
    private List<DownloaderFile> downloaderFileList;
    private Timer timer;
    private Downloader downloader;
    
    /** Creates a new instance of WizardPanelDownload */
    public WizardPanelDownload() {
        super(cardName);
        this.setTitle("Download");
        this.setDescription("Please wait while downloading all files...");
        
        globalStatus = new JLabel("Connecting...");
        globalStatus.setAlignmentX(0.0f);
        
        globalProgressBar = new JProgressBar();
        globalProgressBar.setAlignmentX(0.0f);
        
        currentFileStatus = new JLabel("Current file : xxx");
        currentFileStatus.setAlignmentX(0.0f);
        
        currentFileProgressBar = new JProgressBar();
        currentFileProgressBar.setAlignmentX(0.0f);
        
        messageLabel = new JLabel("Status: ready");
        messageLabel.setFont(messageLabel.getFont().deriveFont(Font.BOLD));
        
        this.contentPanel.add(globalStatus);
        this.contentPanel.add(globalProgressBar);
        this.contentPanel.add(Box.createRigidArea(new Dimension(1, 30)));
        this.contentPanel.add(currentFileStatus);
        this.contentPanel.add(currentFileProgressBar);
        this.contentPanel.add(Box.createRigidArea(new Dimension(1, 30)));
        this.contentPanel.add(messageLabel);
        this.contentPanel.add(Box.createVerticalGlue());
        this.contentPanel.setBorder(BorderFactory.createEmptyBorder(15,15,15,15));
    }

    public void reinit(){
        DownloaderFile.setDefaultTargetDirectory(getApplication().targetDirectory);
        
        if(getApplication().downloadComplete){
            nextButton.setEnabled(true);
            previousButton.setEnabled(true);
            return;
        }
        nextButton.setEnabled(false);
        previousButton.setEnabled(false);
        if(downloader==null){
            downloader = new Downloader(getApplication().downloaderFileList);
        
            initProgressBars();
            
            //start timer         
            int delay = timerRefreshTime; //1sec
            timer = new Timer(delay, this); //the timer permforms an action
            timer.start();

            downloader.downloadAll();
        }

    }
    
    public void actionPerformed(ActionEvent event){
        Object source = event.getSource();
        //Debug.println("action event");
        if(source.equals(cancelButton)){
            Debug.println("clic on cancel button.");
            if(downloader!=null){
                Debug.println("Downloader status="+downloader.getStatus());
                if(downloader.getStatus()==downloader.STATUS_DOWNLOADING){
                    int choice = JOptionPane.showConfirmDialog(
                            wizardFrame,"Are you sure you " +
                            "want to cancel the downloading and quit the application ? ",
                            "Confirmation", 
                            JOptionPane.OK_CANCEL_OPTION, 
                            JOptionPane.WARNING_MESSAGE);
                    if(choice==JOptionPane.OK_OPTION){
                        timer.stop();
                        downloader.interrupt();
                        getApplication().quit();
                        return;
                    }
                }
            }
            getApplication().quit();
        }
        if(source.equals(helpButton)){
            Debug.println("clic on help button.");
        }
        if(source.equals(previousButton)){
            wizardFrame.setCurrentPanel(WizardPanelTargetDir.cardName);
        }
        if(source.equals(nextButton)){
            if(getApplication().downloadComplete){
                wizardFrame.setCurrentPanel(WizardPanelViewFiles.cardName);
            }
        }
        if(source.equals(timer)){
            checkStatus();  
        }
    }  
    
       public void checkStatus(){
        //when download is finish
        //when error occurs
        int status = downloader.getStatus();
        refreshProgressBars();
        long total = downloader.getTotalDownloadedFile();
        if(status==downloader.STATUS_FINISH){
            timer.stop();
            globalProgressBar.setValue(100);
            messageLabel.setText("Status: download complete.");
            nextButton.setEnabled(true);
            previousButton.setEnabled(true);
            getApplication().downloadComplete=true;
            downloader=null;
            
            //DUMP HASH
            {
                Iterator<DownloaderFile> it = getApplication().downloaderFileList.iterator();
                while(it.hasNext()){
                    DownloaderFile df = it.next();
                    byte[] digest = df.getDigest("SHA-1");
                    System.out.printf("File: %s\n  ->SHA1 (base64)=%s\n",df.getRelativePath(),
                                be.cardon.utils.Base64Utils.base64Encode(digest));
                }
            }
            return;
        }
        if(status==downloader.STATUS_ERROR){
            timer.stop();
            this.globalProgressBar.setIndeterminate(false);
            this.currentFileProgressBar.setIndeterminate(false);
            messageLabel.setText("Status: download error!");
            Exception lastDownloadException = downloader.getLastException();
            DialogBox.showExceptionBox(wizardFrame, "Download error", 
                    "Download error", lastDownloadException.getMessage(), 
                    lastDownloadException);
            downloader=null;
            previousButton.setEnabled(true);
            return;
        }
    } 
    public void initProgressBars(){
            globalProgressBar.setMinimum(0);
            currentFileProgressBar.setMinimum(0);
            globalProgressBar.setMaximum(100);
            currentFileProgressBar.setMaximum(100);
            
    }
    
    public void refreshProgressBars(){
            if(downloader==null){
                Debug.println("Downloader = null");
                return;
            }
            //total
            long total = downloader.getTotalDownloadedFile();
            long totalMax = downloader.getTotalLength();

            if(totalMax==-1){
                globalStatus.setText(
                        String.format("%s downloaded.",getFriendlySize(total)));
            }else{
                int totalPourcent = getPourcentage(total, totalMax);
                globalProgressBar.setValue(totalPourcent);
            globalStatus.setText(
                    String.format("%s downloaded (total: %s)",
                    getFriendlySize(total),
                    getFriendlySize(totalMax)));
            }
            long current = downloader.getCurrentFileDownloaded();
            long currentMax = downloader.getCurrentFileLength();
            String path;
            try{
            path = downloader.getCurrentDownloaderFile().getRelativePath().getPath();
            }catch(Exception e){
                path = "[no file]";
            }

            if(currentMax ==-1){
                currentFileProgressBar.setValue(0);
                currentFileStatus.setText(
                        String.format("File %s: %s downloaded.",
                        path, getFriendlySize(current)));
            }else{
            int currentFilePourcent = getPourcentage(current, currentMax);
            currentFileProgressBar.setValue(currentFilePourcent);
            currentFileStatus.setText(
                    String.format("File %s: %s downloaded (total: %s).", 
                    path, getFriendlySize(current), getFriendlySize(currentMax)));
            }
    }
    

    
   static public String getFriendlySize(long SizeInBytes){
        return be.cardon.utils.FileUtils.getFriendlySize(SizeInBytes);
   }
   static public int getPourcentage(long value, long max){
       if(value==0){
           return 0;
       }
       if(max==-1 || max==0){
           return 0;
       }
       int result = new Double(new Double(value) / new Double(max) * 100).intValue();
       return result;
       }
       
   }
