/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.wizard;

import be.cardon.utils.Debug;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.io.Writer;
import java.security.PrivateKey;
import be.cardon.sign.shared.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.io.File;
import java.security.KeyStore;
import be.cardon.sign.shared.DialogBox;

/**
 *
 * @author Rodolphe
 */
public class WizardPanelSelectKey extends WizardDefaultPanel
implements TreeSelectionListener{

    public static String cardName = "keyPanel";

    private JTree providerTree;
    private Box providerRightPanel;
    private KeyStoreTable keyStoreTable; //current visible table
    private final static String MS_CRYPTO_ID = "mscrypto";
    private final static String JKS_ID = "jks";
    private final static String PKCS12_ID = "pkcs12";
    private final static String PKCS11_ID = "pkcs11";
    //private boolean _init=false;
    
    
    /** Creates a new instance of WizardPanelSelectKey */
    public WizardPanelSelectKey() {
 
        super(cardName);
        this.setTitle("Private key");
        this.setDescription("Please select your private key !");

        //contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
        
        // tree
        JTree providerTree = providerTree();
        this.providerTree = providerTree;
        GUIUtils.setMaxSizeUnfinite(this.providerTree);
        
        //scroll with tree
        JScrollPane providerScroll = new JScrollPane(providerTree);
        //providerScroll.setPreferredSize(new Dimension(300,300));
        
        this.providerRightPanel = Box.createVerticalBox();
        this.providerRightPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        
        
        
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, providerScroll, this.providerRightPanel);
        splitPane.setPreferredSize(new Dimension(500,300));
        splitPane.setAlignmentX(0.0f);
        splitPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        GUIUtils.setMaxSizeUnfinite(splitPane);
        
        contentPanel.add(splitPane);
        
        //this.contentPanel.add(contentPanel);
        displayRootPanel();

    }
 
        public void reinit(){
        
    }
    
    public void actionPerformed(ActionEvent event){
        Object source = event.getSource();
        if(source.equals(cancelButton)){
            if(this.showDefaultQuitDialog()){
                getApplication().quit();
                return;
            }
        }
        if(source.equals(helpButton)){
            Debug.println("clic on help button.");
        }
        if(source.equals(previousButton)){
            wizardFrame.setCurrentPanel(WizardPanelViewFiles.cardName);
            return;
        }
        if(source.equals(nextButton)){
            if(this.keyStoreTable!=null){
                KeyStoreEntry kse = this.keyStoreTable.getSelectedEntry();
                if(kse!=null){
                    getApplication().signingKeyStoreEntry =kse;
                    this.wizardFrame.setCurrentPanel(WizardPanelSignAndUpload.cardName);
                    return;
                }
            }
            DialogBox.showWarningBox(wizardFrame, "Please select a private key.");
            return;
        }
    }

    
    private JTree providerTree(){
        DefaultMutableTreeNode top =
                new DefaultMutableTreeNode(new TreeEntry("Providers", "root"));
        if(be.cardon.utils.OperatingSystem.isWindows()){
            top.add(new DefaultMutableTreeNode(new TreeEntry("Windows store", MS_CRYPTO_ID)));
        }
        top.add(new DefaultMutableTreeNode(new TreeEntry("Java key store", JKS_ID)));
        top.add(new DefaultMutableTreeNode(new TreeEntry("PKCS#12 File", PKCS12_ID)));
        top.add(new DefaultMutableTreeNode(new TreeEntry("PKCS#11 DLL library", PKCS11_ID)));
        
        JTree provTree = new JTree(top);
        provTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        provTree.addTreeSelectionListener(this);
        
        return provTree;
        
    }
    
    public void valueChanged(TreeSelectionEvent e) {
        Debug.println("KeyPanel : valueChanged (TreeSelectionEvent)");
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
        this.providerTree.getLastSelectedPathComponent();
        
        if (node == null) return;
        
        Object nodeInfo = node.getUserObject();
        if (node.getUserObject() instanceof TreeEntry) {
            Debug.println("KeyPanel : valueChanged, clic on TreeEntry");
            this.keyStoreTable = null; //reset the pointer to keyStoreTable
            TreeEntry treeEntry = (TreeEntry)node.getUserObject();
            String id = treeEntry.id;
            if(id=="root")      {displayRootPanel(); return; }
            if(id==MS_CRYPTO_ID){displayMSCryptoPanel(); return;}
            if(id==this.JKS_ID){displayJKSPanel(); return;}
            if(id==this.PKCS11_ID){displayPKCS11Panel(); return;}
            if(id==this.PKCS12_ID){displaPKCS12Panel(); return;}
        } else {
            //nothing
        }
    }
    
    
    /**when Root node is selected.*/
    private void displayRootPanel(){
        JPanel descriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        //descriptionPanel.setAlignmentX(0.0f);
        JTextArea descr = GUIUtils.newTextAreaNote(this,"Select a provider.");
        descr.setAlignmentX(0.0f);
        descriptionPanel.add(descr);
        
        providerRightPanel.removeAll();
        providerRightPanel.add(descriptionPanel);
        providerRightPanel.revalidate();
        providerRightPanel.repaint();
    }
    /**not implemented.*/
    private void displayNotImplementedPanel(){
        JPanel descriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        //descriptionPanel.setAlignmentX(0.0f);
        JTextArea descr = GUIUtils.newTextAreaNote(this,"Not implemented.");
        descr.setAlignmentX(0.0f);
        descriptionPanel.add(descr);
        
        providerRightPanel.removeAll();
        providerRightPanel.add(descriptionPanel);
        providerRightPanel.revalidate();
        providerRightPanel.repaint();
    }

    private void displayMSCryptoPanel(){
        Debug.println("KeyPanel: displayMSCryptoPanel");
        try{
            JLabel description = new JLabel("Content of 'My' store.");
            description.setAlignmentX(0.0f);
            
            KeyStore keystore = KeyStoreUtils.getMSKeyStore();
            KeyStoreTable keyStoreTable = new KeyStoreTable(keystore);
            this.keyStoreTable = keyStoreTable;
            JScrollPane scroll= new JScrollPane(keyStoreTable);
            scroll.setAlignmentX(0.0f);
            providerRightPanel.removeAll();
            providerRightPanel.add(description);
            providerRightPanel.add(scroll);
            revalidateAndRepaint(providerRightPanel);
        }catch(Exception e){
            DialogBox.showExceptionBox(wizardFrame, e);
            providerRightPanel.removeAll();
            providerRightPanel.add(new JLabel("Unable to access the keyStore."));
        }
    }
    private void displayJKSPanel(){
        JPanel jksPanel= new JKSPanel();
        providerRightPanel.removeAll();
        providerRightPanel.add(jksPanel);
        revalidateAndRepaint(providerRightPanel);
        
    }
    private void displaPKCS12Panel(){
        JPanel jksPanel= new PKCS12Panel();
        providerRightPanel.removeAll();
        providerRightPanel.add(jksPanel);
        revalidateAndRepaint(providerRightPanel);        
        
    }
    private void displayPKCS11Panel(){
        JPanel jksPanel= new PKCS11Panel();
        providerRightPanel.removeAll();
        providerRightPanel.add(jksPanel);
        revalidateAndRepaint(providerRightPanel);       
        
    }
    
    // ********************** JKS PANEL *******************************
    
    class JKSPanel extends JPanel implements ActionListener{
        
        private JButton defaultFileButton;
        private JButton selectFileButton;
        private JLabel filePath;
        private KeyStoreTable ksTable;
        private Box ksBox; //will contained the scrollPane with the list
        
        public JKSPanel(){
            Debug.println("KeyPanel: displayJKSPanel");
            this.defaultFileButton = new JButton("Default JKS");
            defaultFileButton.addActionListener(this);
            this.selectFileButton = new JButton("Select JKS File...");
            selectFileButton.addActionListener(this);
            
            Box buttonsBox = Box.createHorizontalBox();
            buttonsBox.add(defaultFileButton);
            buttonsBox.add(selectFileButton);
            buttonsBox.add(Box.createHorizontalGlue());
            
            filePath = new JLabel("No file selected.");
            
            Box fileBox = Box.createHorizontalBox();
            fileBox.add(filePath);
            fileBox.add(Box.createHorizontalGlue());
            
            this.ksBox = Box.createHorizontalBox();
            
            this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
            this.add(buttonsBox);
            this.add(fileBox);
            this.add(this.ksBox);
            
            
        }
        
        public void actionPerformed(ActionEvent e){
            if(e.getSource()==defaultFileButton){
                try{
                File JKSFile = KeyStoreUtils.getDefaultJKSKeyStore();
                if(JKSFile == null){
                    DialogBox.showWarningBox(wizardFrame, "Unable to determines the default Java KeyStore");
                    return;
                }
                this.openJKSFileStore(JKSFile);
                }catch(Exception ex){
                    DialogBox.showExceptionBox(wizardFrame, ex);
                    return;
                }
            }
            if(e.getSource()==selectFileButton){
                File JKSFile = this.JKSfileChooser();
                if(JKSFile==null){return;} //user cancel
                openJKSFileStore(JKSFile);
            }
        }
        public File JKSfileChooser(){
            JFileChooser jfc = new JFileChooser();
            jfc.setDialogTitle("Select a JKS store");
            jfc.setDialogType(JFileChooser.OPEN_DIALOG);
            jfc.setMultiSelectionEnabled(false);
            //TO DO : extension filter
            int status = jfc.showDialog(wizardFrame, "Open JKS store");
            if(status == jfc.APPROVE_OPTION){
                return jfc.getSelectedFile();
            }else{
                return null;
            }
        }
        
        public void openJKSFileStore(File JKSFile){
            try{
                if(JKSFile==null){
                    Debug.println("KeyPane: openJKSFileStore: JKSfile is null");
                    return;
                }
                KeyStore ks=null;
                //attemp to open the store without password.
                ks = KeyStoreUtils.getJKSStore(JKSFile, null);
                if(ks==null){
                    Debug.println("KeyPanel: wrong pass 1");
                    //open dialog
                    boolean passUnvalid = true;
                    boolean firstAsk = true;
                    while(ks==null){
                        char[] pass = DialogBox.askPassword(
                                wizardFrame, "Enter the keystore password.",firstAsk);
                        if(pass==null){return;} //user cancelled
                        firstAsk = false;
                        ks = KeyStoreUtils.getJKSStore(JKSFile, pass);
                    }
                }
                if(ks==null){
                    Debug.println("keystore : null pointer");
                    return;
                }
                //here valid password
                KeyStoreTable ksTable = new KeyStoreTable(ks);
                keyStoreTable = ksTable;
                ksBox.removeAll();
                ksBox.add(new JScrollPane(ksTable));
                filePath.setText("File: "+JKSFile.getAbsolutePath());
                ksBox.revalidate();
                ksBox.repaint();
                
            }catch(Exception xmlEx){
                DialogBox.showExceptionBox(wizardFrame, xmlEx);
                ksBox.removeAll();
                ksBox.add(new JLabel("Unable to access the JKS keyStore."));
            }
        }
    };
    
       // ********************** PKCS12 PANEL *******************************
    
    class PKCS12Panel extends JPanel implements ActionListener{
        
        private JButton selectFileButton;
        private JLabel filePath;
        private KeyStoreTable ksTable;
        private Box ksBox; //will contained the scrollPane with the list
        
        public PKCS12Panel(){
            Debug.println("KeyPanel: PKCS12Panel");
            this.selectFileButton = new JButton("Select PKCS#12 File...");
            selectFileButton.addActionListener(this);
            
            Box buttonsBox = Box.createHorizontalBox();
            buttonsBox.add(selectFileButton);
            buttonsBox.add(Box.createHorizontalGlue());
            
            filePath = new JLabel("No file selected.");
            
            Box fileBox = Box.createHorizontalBox();
            fileBox.add(filePath);
            fileBox.add(Box.createHorizontalGlue());
            
            this.ksBox = Box.createHorizontalBox();
            
            this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
            this.add(buttonsBox);
            this.add(fileBox);
            this.add(this.ksBox);
            
            
        }
        
        public void actionPerformed(ActionEvent e){
            if(e.getSource()==selectFileButton){
                File PKCS12File = this.PKCS12fileChooser();
                if(PKCS12File==null){return;} //user cancel
                openPKCS12FileStore(PKCS12File);
            }
        }
        public File PKCS12fileChooser(){
            JFileChooser jfc = new JFileChooser();
            jfc.setDialogTitle("Select a PKCS#12 file");
            jfc.setDialogType(JFileChooser.OPEN_DIALOG);
            jfc.setMultiSelectionEnabled(false);
            //TO DO : extension filter
            int status = jfc.showDialog(wizardFrame, "Open PKCS#12 file");
            if(status == jfc.APPROVE_OPTION){
                return jfc.getSelectedFile();
            }else{
                return null;
            }
        }
        
        public void openPKCS12FileStore(File PKCS12File){
            try{
                if(PKCS12File==null){
                    Debug.println("KeyPane: openPKCS12FileStore: PKCS12File is null");
                    return;
                }
                KeyStore ks=null;
                //attemp to open the store without password.
                // ks = KeyStoreUtils.getPKCS12Store(PKCS12File, null); => BUG 
                   // with PKCS12 SUN Provider, getCertificate(alias) returns null
                   // if null storepass given
                if(ks==null){
                    Debug.println("KeyPanel: wrong pass 1");
                    //open dialog
                    boolean passUnvalid = true;
                    boolean firstAsk = true;
                    while(ks==null){
                        char[] pass = DialogBox.askPassword(
                                wizardFrame, "Enter the keystore password.",firstAsk);
                        if(pass==null){return;} //user cancelled
                        firstAsk = false;
                        ks = KeyStoreUtils.getPKCS12Store(PKCS12File, pass);
                    }
                }
                if(ks==null){
                    Debug.println("keystore : null pointer");
                    return;
                }
                //here valid password
                KeyStoreTable ksTable = new KeyStoreTable(ks);
                keyStoreTable = ksTable;
                ksBox.removeAll();
                ksBox.add(new JScrollPane(ksTable));
                filePath.setText("File: "+PKCS12File.getAbsolutePath());
                ksBox.revalidate();
                ksBox.repaint();
                
            }catch(Exception xmlEx){
                DialogBox.showExceptionBox(wizardFrame, xmlEx);
                ksBox.removeAll();
                ksBox.add(new JLabel("Unable to access the PKCS#12 keyStore."));
            }
        }
    };
    
           // ********************** PKCS11 PANEL *******************************
    class PKCS11Panel extends JPanel implements ActionListener{
        
        private JButton selectFileButton;
        private JLabel filePath;
        private KeyStoreTable ksTable;
        private Box ksBox; //will contained the scrollPane with the list
        
        public PKCS11Panel(){
            Debug.println("KeyPanel: PKCS11Panel");
            this.selectFileButton = new JButton("Select PKCS#11 Library...");
            selectFileButton.addActionListener(this);
            
            Box buttonsBox = Box.createHorizontalBox();
            buttonsBox.add(selectFileButton);
            buttonsBox.add(Box.createHorizontalGlue());
            
            filePath = new JLabel("No library selected.");
            
            Box fileBox = Box.createHorizontalBox();
            fileBox.add(filePath);
            fileBox.add(Box.createHorizontalGlue());
            
            this.ksBox = Box.createHorizontalBox();
            
            this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
            this.add(buttonsBox);
            this.add(fileBox);
            this.add(this.ksBox);
            
            
        }
        
        public void actionPerformed(ActionEvent e){
            if(e.getSource()==selectFileButton){
                File PKCS11LibFile = this.PKCS11libraryChooser();
                if(PKCS11LibFile==null){return;} //user cancel
                openPKCS11KeyStore(PKCS11LibFile);
            }
        }
        public File PKCS11libraryChooser(){
            JFileChooser jfc = new JFileChooser();
            jfc.setDialogTitle("Select a PKCS#11 library");
            jfc.setDialogType(JFileChooser.OPEN_DIALOG);
            jfc.setMultiSelectionEnabled(false);
            //TO DO : extension filter
            int status = jfc.showDialog(wizardFrame, "Load PKCS#11 library");
            if(status == jfc.APPROVE_OPTION){
                return jfc.getSelectedFile();
            }else{
                return null;
            }
        }
        
        public void openPKCS11KeyStore(File PKCS11LibFile){
            try{
                if(PKCS11LibFile==null){
                    Debug.println("KeyPane: openPKCS12FileStore: PKCS12File is null");
                    return;
                }
                KeyStore ks=null;
                //attemp to open the store without password.
                String tokenName = "webSign";
                ks = KeyStoreUtils.getPKCS11Store(PKCS11LibFile, null, tokenName);
                if(ks==null){
                    Debug.println("KeyPanel: wrong pass 1");
                    //open dialog
                    boolean passUnvalid = true;
                    boolean firstAsk = true;
                    while(ks==null){
                        char[] pass = DialogBox.askPassword(
                                wizardFrame, "Enter the keystore password.",firstAsk);
                        if(pass==null){return;} //user cancelled
                        firstAsk = false;
                        ks = KeyStoreUtils.getPKCS11Store(PKCS11LibFile, pass, tokenName);
                    }
                }
                if(ks==null){
                    Debug.println("keystore : null pointer");
                    return;
                }
                //here valid password
                KeyStoreTable ksTable = new KeyStoreTable(ks);
                keyStoreTable = ksTable;
                ksBox.removeAll();
                ksBox.add(new JScrollPane(ksTable));
                filePath.setText("Library: "+PKCS11LibFile.getAbsolutePath());
                ksBox.revalidate();
                ksBox.repaint();
                
            }catch(Exception xmlEx){
                DialogBox.showExceptionBox(wizardFrame, xmlEx);
                ksBox.removeAll();
                ksBox.add(new JLabel("Unable to access the PKCS#11 keyStore."));
            }
        }
    };
    // **********************                *******************************
    
    
    /** revalidate + repaint component.*/
    private void revalidateAndRepaint(JComponent comp){
        comp.revalidate();
        comp.repaint();
    }

    
}
