/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 */

package be.cardon.sign.wizard;


import be.cardon.utils.Debug;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import be.cardon.sign.shared.DialogBox;

/**
 *
 * @author Rodolphe
 */
public class WizardPanelSignAndUpload extends WizardDefaultPanel
implements Runnable{
    
    public static String cardName = "signAndUploadPanel";
    private Icon emptyIcon;
    private Icon doneIcon;
    private Icon currentActionIcon;
    private JLabel signatureLabel;
    private JLabel uploadLabel;
    private JLabel statusLabel;
    private boolean uploadOK=false;
    /** Creates a new instance of WizardPanelHello */
    
    /** 2Creates a new instance of WizardPanelSignAndUpload */
    public WizardPanelSignAndUpload() {
        super(cardName);
        this.setTitle("Sign and upload.");
        this.setDescription("Signature is processing. The signature is being uploaded to the server.");
        
        JPanel contentPanel = this.contentPanel;
        contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
        contentPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        
        emptyIcon = this.getImageIcon("images/blank.gif","emtpy");
        doneIcon =  this.getImageIcon("images/done.gif","done");
        currentActionIcon =  this.getImageIcon("images/current.gif","current");
        
        signatureLabel = new JLabel("Signature");
        signatureLabel.setIcon(currentActionIcon);
        uploadLabel = new JLabel("Upload");
        uploadLabel.setIcon(emptyIcon);
        statusLabel = new JLabel("Status: Ready.");
        statusLabel.setFont(statusLabel.getFont().deriveFont(Font.BOLD));
        
        contentPanel.add(signatureLabel);
        contentPanel.add(uploadLabel);
        contentPanel.add(Box.createRigidArea(new Dimension(1,30)));
        contentPanel.add(statusLabel);
        
    }
    public void reinit(){
        previousButton.setEnabled(false);
        if(uploadOK == false){
            Thread thread = new Thread(this);
            thread.start();
        }     
    }
    
    //start signature and upload
    public void run(){   
        //refresh gui
        this.statusLabel.setText("Status: signing...");
        this.statusLabel.revalidate();
        
        //signature
        try{
            this.getApplication().sign();
        }catch(Exception e){
            DialogBox.showExceptionBox(this.wizardFrame, "Signature failed", e);
            this.previousButton.setEnabled(true);
            return;
        }
        
        //refresh gui
        signatureLabel.setIcon(this.doneIcon);
        uploadLabel.setIcon(this.currentActionIcon);
        signatureLabel.revalidate();
        uploadLabel.revalidate();
        this.statusLabel.setText("Status: uploading signature...");
        this.statusLabel.revalidate();
        
        //upload
        try{
            int result = this.getApplication().uploadSignature();
            this.uploadOK = true;
            //result ...
        }catch(Exception e){
            DialogBox.showExceptionBox(this.wizardFrame, "Upload failed (to do : try again button)", e);
            this.previousButton.setEnabled(true);
            return;
        }
        
        //refresh gui
        uploadLabel.setIcon(this.doneIcon);
        uploadLabel.revalidate();
        this.statusLabel.setText("Status: operation successfuly completed !");
        this.statusLabel.revalidate();
        this.nextButton.setEnabled(false);
        this.previousButton.setEnabled(false);
        this.cancelButton.setText("Close");
        this.cancelButton.setIcon(null);
        
    }
    
    
    public void actionPerformed(ActionEvent event){
        Object source = event.getSource();
        if(source.equals(cancelButton)){
            if(this.uploadOK){
                //URL refresh
                try{
                    this.getApplication().refreshPage();
                }catch(Exception e){
                    DialogBox.showExceptionBox(this.wizardFrame, e);
                }
                return;
            }
            if(this.showDefaultQuitDialog()){
                getApplication().quit();
                return;
            }
        }
        if(source.equals(helpButton)){
            Debug.println("clic on help button.");
        }
        if(source.equals(previousButton)){
            if(uploadOK){
                Debug.println("previousButton error: upload is done. Cannot returns back.");
            }
            this.wizardFrame.setCurrentPanel(WizardPanelSelectKey.cardName);
        }
        if(source.equals(nextButton)){
            //check appletParameter
            Debug.println("clic on next button");
            return;
            
        }
    }
    
}
