/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.wizard;

import be.cardon.utils.Path;
import be.cardon.utils.Debug;
import be.cardon.utils.OperatingSystem;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import javax.swing.text.*;
import java.io.IOException;
import be.cardon.sign.shared.DialogBox;
/**
 *
 * @author Rodolphe
 */
public class WizardPanelTargetDir extends WizardDefaultPanel implements DocumentListener{
    
    
    public static String cardName = "targetDirectoryPanel";
    
    private JTextField targetDirField;
    private JTextField targetSignatureField;
    private JButton browseButton;
    private JLabel statusLabel;
    
    /** Creates a new instance of WizardPanelTargetDir */
    public WizardPanelTargetDir() {
        super(cardName);
        this.setTitle("Target directory");
        this.setDescription("Select a empty directory where the files will be" +
                " downloaded.");
        
        JLabel targetDirLabel = new JLabel("Target directory:");
        JLabel targetSignatureLabel = new JLabel("Target signature:");
        targetDirLabel.setAlignmentX(0.0f);
        targetSignatureLabel.setAlignmentX(0.0f);
        
        targetDirField = new JTextField();
        targetDirField.setPreferredSize(new Dimension(500, 20));
        targetDirField.setMaximumSize(new Dimension(500, 20));
        targetDirField.setAlignmentX(0.0f);
        Document fieldDoc = targetDirField.getDocument();
        if(fieldDoc!=null){
            Debug.println("addDocumentListener");
            fieldDoc.addDocumentListener(this);
        }else{
            Debug.println("WizardPanelTargetDir: Document fieldDoc is null");
        }
        browseButton = new JButton("Browse...");
        browseButton.addActionListener(this);
        browseButton.setAlignmentX(0.0f);
        
        targetSignatureField = new JTextField();
        targetSignatureField.setPreferredSize(new Dimension(300,20));
        targetSignatureField.setMaximumSize(new Dimension(300,20));
        targetSignatureField.setAlignmentX(0.0f);
        
        statusLabel = new JLabel();
        statusLabel.setAlignmentX(0.0f);
        
        File defaultEmptyFile = getFirstNonExistentDir(null);
        if((defaultEmptyFile!=null)){
            targetDirField.setText(defaultEmptyFile.getAbsolutePath());
        }
        
        contentPanel.add(targetDirLabel);
        contentPanel.add(targetDirField);
        contentPanel.add(browseButton);
        //contentPanel.add(Box.createRigidArea(new Dimension(1,50)));
        //contentPanel.add(targetSignatureLabel);
        //contentPanel.add(targetSignatureField);
        contentPanel.add(statusLabel);
        contentPanel.add(Box.createVerticalGlue());
        contentPanel.setBorder(BorderFactory.createEmptyBorder(25,25,15,15));
    }
    
    /** try to find an non existing dir. Returns null if failed.*/
    public File getFirstNonExistentDir(File parentDirectory){
        int max = 100; //number of tests
        if(parentDirectory==null){
            //use FileChooser to retrives the 'My documents' directory (better
            //than System property user.dir that gives 'c:\\documents and settins\\username...'
            if(OperatingSystem.isWindows()){
                try{
                    parentDirectory = new JFileChooser().getCurrentDirectory();
                }catch(Exception e){
                    Debug.println("WizardPanelTargetDir: security Exception while reading " +
                            " Windows home directory");
                    
                }
            }
            if(parentDirectory==null){
                //linux or windows if filechooser failed
                String parentDirectoryString=null;
                try{
                    parentDirectoryString = System.getProperty(OperatingSystem.USER_HOME_PROPERTY);
                }catch(Exception e){
                    Debug.println("WizardPanelTargetDir: security Exception while reading " +
                            " the home directory");
                }
                if(parentDirectoryString!=null){
                    parentDirectory = new File(parentDirectoryString);
                }
            }
            if(parentDirectory==null){
                Debug.println("WizardPanelTargetDir: unable to find the user home dir");
                return null;
            }
        }
        int counter = 0;
        while(counter<max){
            String dirName;
            if(counter==0){
                dirName = "newDirectory";
            }else{
                dirName = "newDirectory" + Integer.toString(counter);
            }
            File dir = new File(parentDirectory, dirName);
            if(isDirectoryEmptyOrInexist(dir)){
                return dir;
            }
            counter++;
        }
        Debug.println("WizardPanelTargetDir: getFirstNonExistentDir: non existent directory" +
                " not found after "+max+" tests.");
        return null;
    }
    
    public void updateStatusFileExist(){
        String dir = targetDirField.getText();
        if(dir.equals("")){
            statusLabel.setText("");
            return;}
        try{
            File dirFile = new File(dir);
            if(dirFile.exists()){
                if(dirFile.isFile()){
                    statusLabel.setText("File with same path already exists.");
                    return;
                }else{
                    statusLabel.setText("Directory already exists.");
                    return;
                }
            }
            statusLabel.setText("");
        }catch(Exception e){
            e.printStackTrace();
            statusLabel.setText("Invalid directory name.");
            return;
        }
    }
    
    public void reinit(){
        //nextButton.setEnabled(false);
        if(!getApplication().downloadComplete){
                targetDirField.setEnabled(true);
                browseButton.setEnabled(true);
        }else{
                targetDirField.setEnabled(false);
                browseButton.setEnabled(false);
                nextButton.setEnabled(true);
        }
        if(getApplication().signature!=null){
            targetSignatureField.setEnabled(false);
        }else{
            targetSignatureField.setEnabled(true);
        }
    }
    
    public void actionPerformed(ActionEvent event){
        Object source = event.getSource();
        if(source.equals(cancelButton)){
            if(this.showDefaultQuitDialog()){
                getApplication().quit();
                return;
            }
        }
        if(source.equals(helpButton)){
            Debug.println("clic on help button.");
            return;
        }
        if(source.equals(previousButton)){
            this.wizardFrame.setCurrentPanel(WizardPanelHello.cardName);
            return;
        }
        if(source.equals(nextButton)){
            if(getApplication().downloadComplete){
                this.wizardFrame.setCurrentPanel(WizardPanelDownload.cardName);
                return;
            }
            String dirString = targetDirField.getText();
            if(dirString.length()==0){
                DialogBox.showWarningBox(this.wizardFrame, "Select first a directory.");
                return;
            }
            try{
                File dir = new File(dirString);
                if(!isDirectoryEmptyOrInexist(dir)){
                    DialogBox.showWarningBox(this.wizardFrame, "Choose a non existent directory.");
                    return;
                }
                try{createDirectory(dir);} catch(Exception e){
                    DialogBox.showExceptionBox(this.wizardFrame, "Unable to create a new directory.",e);
                    return;
                }
                getApplication().targetDirectory= dir;
                this.wizardFrame.setCurrentPanel(WizardPanelDownload.cardName);
                return;
                
                
            }catch(Exception e){
                e.printStackTrace();
                DialogBox.showExceptionBox(this.wizardFrame, e);
                return;
            }
            
        }
        if(source.equals(browseButton)){
            File targetDirectory = TargetDirectoryChooser();
            if(targetDirectory==null){
                return;
            } else{
                if(targetDirectory.exists()){
                    File emptyTargetDirectory = null;
                    if(targetDirectory.isDirectory()){
                        emptyTargetDirectory = getFirstNonExistentDir(targetDirectory);
                    }else{
                        //impossible if correct file chooser
                        Debug.println("WizardPanelTargetDir: FileChooser returned a file!");
                    }
                    if(emptyTargetDirectory!=null){
                        targetDirectory = emptyTargetDirectory;
                    }
                }
                this.targetDirField.setText(targetDirectory.getAbsolutePath());
                updateStatusFileExist();
                return;
                
                
            }
        }
    }
    
    private boolean isDirectoryEmptyOrInexist(File dir){
        if(!dir.exists()){
            return true;
        }
        if(!dir.isDirectory()){
            return false;
        }
        String[] content = dir.list();
        if(content.length==0){
            return true;
        }else{
            return false;
        }
    }
    
    private void  createDirectory(File dir) throws java.io.IOException{
        if(dir.exists()){return;}
        String separator = Path.getSystemSeparator();

        if(!dir.mkdirs()){
                throw new IOException("Creation of new directory "+dir.getAbsolutePath()+" failed.");
        }
    }
    
    /**Document listener.*/
    public void changedUpdate(DocumentEvent e){
        Debug.println("ChangeUpdate");
        updateStatusFileExist();
    }
    public void insertUpdate(DocumentEvent e) {
        Debug.println("insertUpdate");
        updateStatusFileExist();
    }
    public void removeUpdate(DocumentEvent e){
        Debug.println("removeUpdate");
        updateStatusFileExist();
    }
    
    
    
    /**Return the file or null if user cancelled the operation.*/
    public File TargetDirectoryChooser(){
        try{
            JFileChooser jfc = new JFileChooser();
            jfc.setDialogTitle("Target directory");
            jfc.setDialogType(JFileChooser.SAVE_DIALOG);
            jfc.setMultiSelectionEnabled(false);
            jfc.setFileSelectionMode(jfc.DIRECTORIES_ONLY);
            int status = jfc.showDialog(this.wizardFrame, "OK");
            if(status == jfc.APPROVE_OPTION){
                return jfc.getSelectedFile();
            }else{
                return null;
            }
        }catch(Exception e){
            DialogBox.showExceptionBox(this.wizardFrame, "Can't open the File Chooser" +
                    " (maybe for Security problem).", e);
            return null;
        }
    }
}
