/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 */

package be.cardon.sign.wizard;

import be.cardon.utils.Debug;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import be.cardon.sign.shared.FileHierarchyTreeModel;
import be.cardon.sign.shared.FileHierarchyEntry.*;
import be.cardon.sign.shared.FileHierarchyEntry;
import be.cardon.sign.shared.FileHierarchyTableModel;
import be.cardon.sign.shared.FileHierarchyException;
import be.cardon.sign.shared.FileHierarchyHandler;
import be.cardon.sign.shared.DialogBox;
import java.util.Iterator;
import java.io.File;
import be.cardon.sign.shared.GUIUtils;
import be.cardon.sign.communicator.DownloaderFile;
/**
 *
 * @author Rodolphe
 */
public class WizardPanelViewFiles extends WizardDefaultPanel{
    
    public static String cardName = "viewFilesPanel";
    private JTree fileTree;
    private JTable fileTable;
    private FileHierarchyTreeModel treeModel;
    private JButton propertiesButton;
    private JButton openButton;
    private FileHierarchyTableModel tableModel;
    private WizardPanelViewFiles thisPanel;
    
    
    /** Creates a new instance of WizardViewFiles */
    public WizardPanelViewFiles(){
        
        super(cardName);
        thisPanel = this;
        this.setTitle("Documents view");
        this.setDescription("This window contains the document that will be signed. " +
                " Please read carrefully all the documents before continue.");
        
        fileTree = new JTree(); //model fixed in reinit.
        JScrollPane treeScroll = new JScrollPane(fileTree);
        //treeScroll.setPreferredSize(new Dimension(300,300));
        
        Box fileViewRightPanel = Box.createVerticalBox();
        //fileViewRightPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                treeScroll,fileViewRightPanel);
        splitPane.setPreferredSize(new Dimension(500,300));
        splitPane.setAlignmentX(0.0f);
        GUIUtils.setMaxSizeUnfinite(splitPane);
        
        //this.contentPanel.add(contentPanel);
        JPanel ButtonPanel = new JPanel();
        ButtonPanel.setLayout(new BoxLayout(ButtonPanel, BoxLayout.X_AXIS));
        propertiesButton = new JButton("View Properties");
        propertiesButton.addActionListener(this);
        openButton = new JButton("Open file");
        openButton.addActionListener(this);
        openButton.setEnabled(false);
        ButtonPanel.add(openButton);
        ButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        ButtonPanel.add(propertiesButton);
        ButtonPanel.add(Box.createHorizontalGlue());
        ButtonPanel.setAlignmentX(0.0f);
        
        fileTable = new JTable();
        fileTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        fileTable.setPreferredScrollableViewportSize(new Dimension(400,200));
        fileTable.setMaximumSize(new Dimension(500,500));
        fileTable.getSelectionModel().addListSelectionListener(listSelectListener);
        fileTable.addMouseListener(tableMouseListener);
        JScrollPane fileTableScroll = new JScrollPane(fileTable);
        
        
        fileViewRightPanel.add(fileTableScroll);
        
        contentPanel.add(splitPane);
        contentPanel.add(ButtonPanel);
        contentPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        
    }
    
    public void reinit(){
        try{
            if(treeModel==null){
                boolean caseSensitive = false; //security => work on windows & linux
                treeModel = getfileHierarchyTreeModel(getApplication().downloaderFileList,
                        false);
                fileTree.setModel(treeModel);
                
                tableModel = new FileHierarchyTableModel(
                        (AbstractDirectory)treeModel.getRoot());
                fileTable.setModel(tableModel);
                fileTable.getColumnModel().getColumn(0).setPreferredWidth(100);
                fileTable.getColumnModel().getColumn(1).setPreferredWidth(75);
                fileTable.getColumnModel().getColumn(2).setPreferredWidth(225);
                
                new FileHierarchyHandler(this.fileTree, this.tableModel);
            }
        }catch(Exception e){
            DialogBox.showExceptionBox(wizardFrame, e);
        }
    }
    
    
    /** Creates a new instance of FileHierarchyModel */
    private FileHierarchyTreeModel getfileHierarchyTreeModel(
            java.util.List<DownloaderFile> downloaderFiles, boolean caseSensitive)
            throws FileHierarchyException{
        AbstractDirectory root = new AbstractDirectory(null, "root", caseSensitive);
        FileHierarchyTreeModel model = new FileHierarchyTreeModel(root, caseSensitive);
        for(Iterator<DownloaderFile> it = downloaderFiles.iterator();it.hasNext();){
            DownloaderFile df= it.next();
            File file = df.getRelativePath();
            Object property = df;
            long size = df.getSize();
            model.addFileToRoot(file, property, size);
        }
        return model;
    }
    
    public void actionPerformed(ActionEvent event){
        try{
            Object source = event.getSource();
            if(source.equals(cancelButton)){
                if(this.showDefaultQuitDialog()){
                    getApplication().quit();
                    return;
                }
            }
            if(source.equals(helpButton)){
                Debug.println("clic on help button.");
            }
            if(source.equals(previousButton)){
                wizardFrame.setCurrentPanel(WizardPanelDownload.cardName);
                return;
            }
            if(source.equals(nextButton)){
                wizardFrame.setCurrentPanel(WizardPanelSelectKey.cardName);
                return;
            }
            if(source.equals(openButton)){
                ListSelectionModel lsm = fileTable.getSelectionModel();
                if(lsm.isSelectionEmpty()){
                    return;
                }else{
                    int i=lsm.getMinSelectionIndex();
                    FileHierarchyEntry entry = tableModel.getFileHierarchyEntry(i);
                    if(entry instanceof AbstractDirectory){
                        Debug.println("Clic on a directory");
                        return;
                    }else if(entry  instanceof AbstractFile){
                        Object property = entry.getProperty();
                        DownloaderFile downloaderFile = (DownloaderFile)property;
                        File file = downloaderFile.getLocalAbsoluteFile();
                        be.cardon.utils.OperatingSystem.open(file);
                    }
                }
            }
        }catch(Exception e){
            DialogBox.showExceptionBox(this.wizardFrame, e);
        }
    }
    
    private ListSelectionListener listSelectListener = new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
            //Ignore extra messages.
            if (e.getValueIsAdjusting()) return;
            
            ListSelectionModel lsm =
                    (ListSelectionModel)e.getSource();
            if (lsm.isSelectionEmpty()) {
                propertiesButton.setEnabled(false);
                openButton.setEnabled(false);
            } else {
                int selectedRow = lsm.getMinSelectionIndex();
                propertiesButton.setEnabled(true);
                if(lsm.getMinSelectionIndex()!=lsm.getMaxSelectionIndex()){
                    openButton.setEnabled(false);
                }else{
                    //only one item selected
                    openButton.setEnabled(true);
                }
                
            }
        }
    };
    
    MouseListener tableMouseListener = new MouseAdapter(){
        
        public void mouseClicked(MouseEvent e){
            Debug.println("clic : "+e);
            if(e.getClickCount()==2 && e.getButton()==e.BUTTON1){
                ActionEvent event = new ActionEvent(
                        openButton,
                        ActionEvent.ACTION_PERFORMED,
                        "open");
                thisPanel.actionPerformed(event);
            }
        }
    };
}
