/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.sign.xmldsig.dereferencer;

import javax.xml.crypto.*;
import java.net.URI;
import org.w3c.dom.Document;
import be.cardon.utils.Debug;
import com.sun.org.apache.xml.internal.security.signature.XMLSignatureInput;
import javax.xml.crypto.OctetStreamData;
import be.cardon.sign.xmldsig.dereferencer.implementations.*;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.URIDereferencer;


/**
 *
 * @author Rodolphe
 */
public class DefaultDereferencer implements URIDereferencer{
    
    private Document signatureDoc;
    private URIDereferencer builtInDereferencer;
    
    
    /** Creates a new instance of DefaultDereferencer *
     *Need the document that contains the XMLSignature element, for fragment #ref
     */
    public DefaultDereferencer(Document signatureDoc) {
        //register the dereferencer
        this(signatureDoc, true);
    }
    
    public DefaultDereferencer(Document signatureDoc, boolean registerDefaultDereferencer) {
        //register the dereferencer
        ResourceResolver.init();
        if(registerDefaultDereferencer){
            Debug.println("DefaultDereferencer: registering ResolverLocalFilesystem.");
            Debug.println("Canonical name :"+ ResolverLocalFilesystem.class.getCanonicalName());
            ResourceResolver.register(ResolverLocalFilesystem.class.getCanonicalName());
        }
        this.signatureDoc = signatureDoc;
        
        //register the build-in Dereferencer
        this.builtInDereferencer= XMLSignatureFactory.getInstance().getURIDereferencer();
        if(this.builtInDereferencer!=null){
            Debug.println("DefaultDereferencer : Built-in URIDereferencer installed.");
            Debug.println("Canonical name = "+ this.builtInDereferencer.getClass().getCanonicalName());
        }else{
            Debug.println("DefaultDereferencer : Built-in URIDereferencer Unaivalable.");
        }
        
    }
    
    
    /**
     *Dereferences the specified URIReference and returns the dereferenced data.
     *@param uriReference the URIReference
     *@param context an XMLCryptoContext that may contain additional useful
     * information for dereferencing the URI. This implementation should
     * dereference the specified URIReference against the context's baseURI
     * parameter, if specified.
     *@returns the dereferenced data
     *@throws NullPointerException if uriReference or context are null
     *@throws URIReferenceException if an exception occurs while dereferencing
     * the specified uriReference
     */
    public Data dereference(URIReference uriReference, XMLCryptoContext context)
    throws NullPointerException, URIReferenceException{
        Debug.println("DefaultDereference: "+uriReference.getURI());
        if(uriReference==null){
            throw new NullPointerException("uriReference");
        }
        if(context==null){
            throw new NullPointerException("context");
        }
        try{
            XMLSignatureInput xsi = ResourceResolver.resolveStatic(uriReference, context.getBaseURI(), this.signatureDoc);
            
            if(xsi.getOctetStream()==null){
                Debug.println("WARNING DefaultDereferencer octet stream is null. Probably NodeSetData");
                throw new URIReferenceException("NodeSetData conversion not implemented. Only" +
                        "binary stream data can be now used.");
            }
            
            Data data = new OctetStreamData(xsi.getOctetStream(), xsi.getSourceURI(), xsi.getMIMEType());
            return data;
            
        }catch(ResourceResolverException e){
            //use default Resolver
            Debug.println("use default Resolver for "+uriReference.getURI());
            if(builtInDereferencer!=null){
                return builtInDereferencer.dereference(uriReference, context);
            }else{
                throw new URIReferenceException("Dereferencer not found");   
            }
            
        }catch(Exception e){
            throw new URIReferenceException(e);
        }
    }
}
