/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.utils;

import javax.security.auth.x500.X500Principal;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**X500 Parser, returns the CN (Common Name) of a given X500Principal.
 *
 * @author Rodolphe
 */
public class X500Parser {
    
    X500Principal principal;
    List<X500Element> x500elements;
    
    /** Creates a new instance of X500Parser */
    public X500Parser(X500Principal principal){
        this.principal = principal;
        String principalString = principal.toString();
        //parsing
        
        this.x500elements = new ArrayList<X500Element>();
        
        int from = 0;
        while(from < principalString.length()-1){
            int comaIndex = principalString.indexOf(",", from);
            if(comaIndex < 0){
                comaIndex = principalString.length();
            }
            String entry = principalString.substring(from, comaIndex).trim();
            int equalIndex = entry.indexOf("=");
            if(equalIndex < 0){
                System.err.println("Warning : X500Principal Parsing : no " +
                        "equal found in X500 entry");
                break;
            }
            String key = entry.substring(0, equalIndex).trim();
            String value = entry.substring(equalIndex + 1).trim();
            this.x500elements.add(new X500Element(key, value));
            from = comaIndex + 1;
        }
        
    }
    
    /** Returns the first 'CN' value, or the whole DN if CN is not found.*/
    public String getCN(){
        for(Iterator<X500Element> it = this.x500elements.iterator();it.hasNext();){
            X500Element x500el = it.next();
            if(x500el.key.toUpperCase().equals("CN")){
                return x500el.value;
            }
        }
        //CN not found
        Debug.println("CN not found in "+ this.principal.toString());
        return this.principal.toString();
    }
    
    public String toString(){
        String out=new String();
        for(Iterator<X500Element> it = this.x500elements.iterator();it.hasNext();){
            X500Element x500el = it.next();
            String key = x500el.key;
            String value = x500el.value;
            out= out + key + "="+value+"\n"; 
        }
        return out;
    }
    
    class X500Element{
        public String key;
        public String value;
        
        public X500Element(String key, String value){
            this.key = key; 
            this.value = value;
        }
    }
}
